from typing import List
from .prompts import ElementsPromptTemplate,ElementPromptTemplate
from langchain.base_language import BaseLanguageModel
from langchain.chains import LLMChain


class ElementsExtractor:
    def __init__(self, llm: BaseLanguageModel):
        self.prompt = ElementsPromptTemplate(input_variables=["knowledge","elements"])
        self.chain = LLMChain(llm=llm, prompt=self.prompt,verbose=True)
        self.prompt_foreach = ElementPromptTemplate(input_variables=["knowledge","element"])
        self.chain_foreach = LLMChain(llm=llm, prompt=self.prompt_foreach,verbose=True)

    def extract(self, knowledge: str,elements: List[str]) -> List[str]:
        output = self.chain.run({"knowledge":knowledge,"elements":elements})
        lines = [line for line in output.split("\n") if line.strip()]
        return lines
    
    def extract_foreach(self, knowledge: str,elements: List[str]) -> List[str]:
        lines = []
        for e in elements:            
            output = self.chain_foreach.run({"knowledge":knowledge,"element":e})
            lines.append(output)
            print(output)
        return lines