from typing import List
from .prompts import QuestionGeneratePromptTemplate,AnswerPromptTemplate
from langchain.base_language import BaseLanguageModel
from langchain.chains import LLMChain
from langchain.prompts import StringPromptTemplate,PromptTemplate


class QAGenerator:
    def __init__(self, llm: BaseLanguageModel):
        self.prompt = QuestionGeneratePromptTemplate(input_variables=["knowledge","question_number"])
        self.chain = LLMChain(llm=llm, prompt=self.prompt)
        self.answer_prompt = AnswerPromptTemplate(input_variables=["knowledge","question"])
        self.answer_chain = LLMChain(llm=llm, prompt=self.answer_prompt)

    def generate_questions(self, knowledge: str,question_number: int | None=None) -> List[str]:
        output = self.chain.run({"knowledge":knowledge,"question_number":question_number})
        lines = [line for line in output.split("\n") if line.strip() and line.startswith("问：")]
        return lines

    def generate_answer(self, knowledge: str, question: str) -> str:
        answer = self.answer_chain.run({"knowledge":knowledge,"question":question})
        return answer
    
    def generate(self, knowledge: str, question_number=3):
        questions = self.generate_questions(knowledge, question_number)
        if len(questions) == 0:
            return None
        questions = [question.replace("问：","") for question in questions if question.startswith("问：")]    
        answers = []
        for question in questions:
            answer = self.generate_answer(knowledge, question)
            answers.append(answer)
        return [(question,answer) for question,answer in zip(questions,answers)]


prompt_template="""从下面这段话中提取出关键信息，生成 {question_number} 个问题，并回答相关问题。
输出格式为一问一答，问题以"问："开头，答案以"答："开头。
不需要标注问题序号， 问题和答案相互映射，每个问题相互独立。

{knowledge}
"""

class TrainData:
    def __init__(self, llm: BaseLanguageModel):
        self.prompt = PromptTemplate.from_template(prompt_template)
        self.chain = LLMChain(llm=llm, prompt=self.prompt,verbose=False)

    def generate(self, knowledge: str, question_number=3):
        res=self.chain.run({"knowledge":knowledge,"question_number":question_number})
        print(res)
        qas = []
        q, a = None, ''
        lines = res.split("\n")
        for line in lines:
            line = line.strip()
            if line.startswith('问：'):
                if q is not None and a is not None:
                    qas.append((q, a))
                q = line.replace('问：', '')
                a = ''
            elif line.startswith('答：'):
                a += line.replace('答：', '') + '\n'
            elif a is not None:
                a += line + '\n'
        if q is not None and a is not None:
            qas.append((q, a))
        return qas
        # qas = [qa for qa in qas if qa.strip()]
        # questions = [qa for qa in qas if qa.startswith("问：")]
        # answers = [qa for qa in qas if qa.startswith("答：")]
        # if len(questions) != len(answers):
        #     print(ValueError("questions and answers are not equal"))
        #     return []
        # return [(question.strip().replace("问：",""),answer.strip().replace("答：","")) for question,answer in zip(questions,answers)]
        # return [{"Q":question,"A":answer} for question,answer in zip(questions,answers)]
    