
from typing import Any
from langchain.prompts import StringPromptTemplate,PromptTemplate
from pydantic import BaseModel, validator


template="""
{knowledge}
请从上述内容中归纳总结出 {question_number} 个问题，问题以"问："开头，以问号结尾，用空行分隔。
"""
prompt=PromptTemplate.from_template(template)

class QuestionGeneratePromptTemplate(StringPromptTemplate, BaseModel):
        
    def format(self, **kwargs) -> str:
        if "question_number" not in kwargs or kwargs["question_number"] is None:
            question_number = 5
        else:    
            question_number = kwargs["question_number"]
        if "knowledge" not in kwargs:
            raise ValueError("knowledge is required")
        knowledge = kwargs["knowledge"]

        return prompt.format(question_number=question_number, knowledge=knowledge)

    def _prompt_type(self):
        return "question-generator"
    

template_answer="""
请参考以下内容中回答问题：{question}，回答以"答："开头。
下面是你参考的内容：
{knowledge}
"""

class AnswerPromptTemplate(StringPromptTemplate, BaseModel):
    def format(self, **kwargs: Any) -> str:
        if "knowledge" not in kwargs:
            raise ValueError("knowledge is required")
        if "question" not in kwargs:
            raise ValueError("question is required")
        
        return template_answer.format(question=kwargs["question"], knowledge=kwargs["knowledge"])