
from langchain import PromptTemplate
from langchain.base_language import BaseLanguageModel
from langchain.chains import LLMChain


question_rdf_prompt_template="""
你的任务是帮助我重新整理问题，使得问题中的指代信息更加明确，你的输出只会是原问题或者重新整理之后的问题，不包含其他内容。
==============================
{history}
以上是对话历史。
==============================
这是我的问题：
{question}
==============================
如果你认为这个问题中存在不明确的指代信息，请从对话历史中提取出相关信息，重新定义问题并直接输出，不要增加无关内容。
否则，请直接输出原问题。直接输出原问题的格式为：

{question}
"""


question_rdf_prompt=PromptTemplate.from_template(question_rdf_prompt_template)

class QuestionRDF:
    def __init__(self,llm:BaseLanguageModel):
        self.llm=llm
        self.chain=LLMChain(llm=llm,prompt=question_rdf_prompt,verbose=True)

    def generate(self,history:str,question:str):
        res=self.chain.run({"history":history,"question":question})
        return res
