import sys
sys.path.append("../..") 

import gradio as gr
import torch

from contract.extraction import ElementsExtractor
from llm.chatglm import ChatGLMLocLLM
from llm.ernie import ErnieLLM
from llm.baichuan import BaichuanLLM
from loader.load import load_file,load
from common import consts

from flask.cli import load_dotenv
load_dotenv()


from argparse import Namespace

from llm.loader import ModelLoader
cfg = Namespace()

#model
cfg.model_name_or_path = consts.MODEL_PATH_ChatGLM2_32K
cfg.lora_checkpoint = '/home/zfh/aird/model/ckpt/chatglm-6b-lora-spdsvb-INSv8-1e-03-30'
cfg.pre_seq_len = None 
cfg.quantization_bit = None

loader = ModelLoader(cfg.model_name_or_path, cfg.quantization_bit)
# loader.load_lora(cfg.lora_checkpoint)
model,tokenizer = loader.models()
model = model.eval()

# max_length=1000
# Define the Gradio interface
def contract(chatbot, file, input, history, max_length):
    print("contract",file,input,max_length)
    chatbot.append((input, ""))
    if file is not None:
        content = load(file.name)
        content="\n".join([d.page_content for d in content])
        print(len(content))
    if content is not None and len(content) > 0:
        if len(history) == 0:
            prompt = f"{content}请基于上述内容回答以下问题：\n{input}"
        else:
            prompt = input
    else:
        prompt = input
    # print("prompt: ",prompt)
    for response, history in model.stream_chat(tokenizer, prompt, history, max_length=max_length):  # type: ignore
        chatbot[-1] = (input, response) 
        yield chatbot, history

def reset(history):
    history=[]
    file.value=None
    return history

with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">商业合同要素提取</h1>""")
    
    history = gr.State([])
    chatbot = gr.Chatbot()

        # with gr.Column(scale=4):
    # with gr.Row():
    #     input=gr.Textbox(label="输入文本", type="text", lines=5)
    with gr.Row():
        with gr.Column(scale=4):
            file = gr.File(label="上传文件")
        with gr.Column(scale=4):
            input=gr.Textbox(label="输入文本", type="text", lines=5)
        with gr.Column(scale=4):    
            with gr.Row():
                max_length = gr.Slider(1000, 30000, value=30000, step=1000, label="单次提取使用的文本长度", interactive=True)
            with gr.Row():
                submit_btn=gr.Button("开始提取")
                reset_btn=gr.Button("重置")
                reset_btn.click(reset,inputs=[history],outputs=[history])
                            
    submit_btn.click(contract,inputs=[chatbot, file, input, history, max_length],outputs=[chatbot,history])
            
demo.queue().launch(share=True)