from .c_db import UPostgresDB
import json
TABLE_CHAT = """
create table chat (
    id varchar(1000) primary key, 
    user_id int,
    info text,
    chat_type_id int,
    create_time date,
    history json,
    is_delete int,
    status int
);
"""

class Chat:
    def __init__(self, db: UPostgresDB) -> None:
        self.db = db

    def insert(self, value):
        value[5] = json.dumps(value[5])
        query = f"INSERT INTO chat(id,user_id,info,chat_type_id,create_time,history,is_delete,status) VALUES (%s,%s,%s,%s,%s,%s,%s,%s)"
        self.db.execute_args(query, ((value[0],value[1],value[2],value[3],value[4],value[5],value[6],value[7])))

    def delete_update(self,id):
        query = f"UPDATE chat SET is_delete = 1 WHERE id = %s"
        self.db.execute_args(query, (id,))


    def search(self, id):
        query = f"SELECT chat.id,user_id,info,t.name as type,chat.create_time,history,is_delete,status FROM chat left join chat_type t on t.id=chat.chat_type_id WHERE chat.id = %s and is_delete=0 ORDER BY chat.create_time DESC;"
        self.db.execute_args(query, (id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0]
        else:
            return None
        
    def manual_search(self, id):
        query = f"SELECT user_id,status,history FROM chat WHERE id = %s and is_delete=0"
        self.db.execute_args(query, (id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0]
        else:
            return None

    def qa_search(self, id):
        query = f"SELECT user_id,info,t.name as type,chat.create_time,history,status FROM chat left join chat_type t on t.id=chat.chat_type_id WHERE chat.id = %s and is_delete=0 ORDER BY chat.create_time DESC;"
        self.db.execute_args(query, (id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0]
        else:
            return None
    
    def detail_search(self, id):
        query = f"SELECT chat.id,user_id,info,t.name as type,chat.create_time,history,status FROM chat left join chat_type t on t.id=chat.chat_type_id WHERE chat.id = %s and is_delete=0 ORDER BY chat.create_time DESC;"
        self.db.execute_args(query, (id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0]
        else:
            return None
        
    def delete_search(self, id):
        query = f"SELECT user_id FROM chat WHERE id = %s and is_delete=0"
        self.db.execute_args(query, (id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0]
        else:
            return None

    def list_chat_search(self, chat_type, user_id):
        query = f"SELECT chat.id,info,t.name as type,chat.create_time,history,chat.status FROM chat left join chat_type t on t.id=chat.chat_type_id WHERE t.name = %s AND is_delete=0 AND user_id = %s ORDER BY chat.create_time DESC;" 
        self.db.execute_args(query,  (chat_type, user_id))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer
        else:
            return None

    def search_history(self,id):
        query = f"SELECT history FROM chat WHERE id = %s ORDER BY chat.create_time DESC;"
        self.db.execute_args(query, (id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            answer[0] = answer[0][0]
            return answer[0]
        else:
            return None
        
    def get_last_q(self):
        query = f"SELECT id,create_time,info,chat_type_id FROM chat ORDER BY create_time DESC LIMIT 1 " 
        self.db.execute(query)
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0]
        else:
            return None

    def create_table(self):
        query = f"SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_name = 'chat')"
        self.db.execute(query)
        exists = self.db.fetchall()[0][0]
        if not exists:
            query = TABLE_CHAT
            self.db.execute(query)

    def drop_table(self):
        query = f"SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_name = 'chat')"
        self.db.execute(query)
        exists = self.db.fetchall()[0][0]
        if exists:
            query = "DROP TABLE chat"
            self.db.format(query)
            print("drop table chat ok")

    def update(self, chat_id, history):
        history = json.dumps(history)
        query = f"UPDATE chat SET history = %s WHERE id = %s"
        self.db.execute_args(query, (history,chat_id))

    def history_update(self, chat_id, history):
        history = json.dumps(history)
        query = f"UPDATE chat SET history = %s WHERE id = %s"
        self.db.execute_args(query, (history,chat_id))


    def search_type_id(self, chat_id):
        query = f"select chat_type_id from chat where id = %s"
        self.db.execute_args(query, (chat_id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0][0]
        else:
            return None
        
    def get_chat_status(self, chat_id):
        query = f"Select status from chat where id = %s"
        self.db.execute_args(query, (chat_id,))
        answer = self.db.fetchall()
        if len(answer) > 0:
            return answer[0][0]
        else:
            return None
        
    def set_chat_status(self, chat_id, status):
        query = f"update chat set status = %s where id = %s"
        self.db.execute_args(query, (status,chat_id))