from fastapi import FastAPI, HTTPException, Header
from pydantic import BaseModel
import sys
sys.path.append("../..") 
from llm.chatglm import ChatGLMSerLLM
from langchain import LLMChain
from langchain.vectorstores.faiss import FAISS
app = FastAPI()
import math
from fastapi.middleware.cors import CORSMiddleware
from scenarios.psbc.model_serve import modelcall_prase, CHATGLM_PROMPT
# 添加 CORS 中间件
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # 允许所有域访问，也可以指定特定域名
    allow_credentials=True,
    allow_methods=["*"],  # 允许所有HTTP方法
    allow_headers=["*"],  # 允许所有HTTP头
)


model_url = "http://192.168.10.93:8000"


def customize_exception(code: int, message: str, data: dict):
    return {
        "code":code,
        "message":message,
        "data":data
    }


@app.get("/query")
async def query(query:str=None):
    # modelcall_prase没有将url和CHATGLM_PROMPT变量封装进去，在这里需要进行声明。
    base_llm=ChatGLMSerLLM(url=model_url)
    chose_llm = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT)
    try:
        result, title, summary, tags, category, subcategories = modelcall_prase(chose_llm, query)
    except Exception as e:
        print("模型调用过程或json解析过程出现错误：", e)
        return customize_exception(40012, str(e), {})
    return customize_exception(200, "success", {
        "result": result,
        "title": title,
        "summary": summary,
        "tags": tags,
        "category": category,
        "subcategories": subcategories
    })


@app.get("/tags")
async def tags(tags_native:list, faiss_vectorstore:FAISS, threshold:float=0.7):
    score_threshold = (1-threshold) * math.sqrt(2)
    tags_new = []
    for tag_native in tags_native:
        res = faiss_vectorstore.similarity_search_with_score(query=tag_native, score_threshold=score_threshold)
        for document_object, _ in res:
            tags_new.append(document_object.page_content)
    return customize_exception(200, "success", {
        "tags_native": tags_native,
        "tags_new": tags_new
    })


if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8888)
