# -*- coding: utf-8 -*-
import os, sys

import pandas as pd
sys.path.append("../..") 

import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate

from typing import Awaitable
import asyncio
from langchain.callbacks  import AsyncIteratorCallbackHandler
import re
from llm.ernie_with_sdk import ChatERNIESerLLM
from qianfan import ChatCompletion


chatglm3_prompt_tfq = """<|system|>
你是一个可以将一段文本根据其内容生成可以用于考试的判断题，并按照一定格式进行输出的工具，一共输出{num_selector}个问题。具体的输出格式如下所示:
试题1：
题目：。
答案：正确/错误。
解析：用到的资料中的内容。

试题2：
题目：。
答案：正确/错误。
解析：用到的资料中的内容。
...
<|user|>
文本内容如下：
{context}"""

chatglm3_prompt_mcq = """<|system|>
你是一个可以将一段文本根据其内容生成可以用于考试的选择题，并按照一定格式进行输出的工具，一共输出{num_selector}个问题。具体的输出格式如下所示:
试题1：
题目：。
A.选项内容    B.选项内容    C.选项内容    D.选项内容
正确答案：给出具体选项。

试题2：
题目：。
A.选项内容    B.选项内容    C.选项内容    D.选项内容
正确答案：给出具体选项。
...
<|user|>
文本内容如下：
{context}"""

qianfan_prompt_tfq = """请根据下面提供的知识资料，生成可以作为考试的判断题，并给出正确答案，一共输出{num_selector}个问题。按照如下格式进行回答：
试题1：
题目：试题内容。
正确答案：正确/错误。
解析：选择原因（如果用到原资料中的内容，请列出来）

试题2：
题目：试题内容。
正确答案：正确/错误。
解析：选择原因（如果用到原资料中的内容，请列出来）

...

资料内容如下所示：
'''
{context}
'''"""

qianfan_prompt_mcq = """请你根据下面这一段文本的内容生成可以用于考试的选择题，并按照一定格式进行输出，一共输出{num_selector}个问题。具体的输出格式如下所示:
试题1：
题目：
A.选项内容
B.选项内容
C.选项内容
D.选项内容
正确答案：给出具体选项。

试题2：
题目：
A.选项内容
B.选项内容
C.选项内容
D.选项内容
正确答案：给出具体选项。
...

文本内容如下：
'''
{context}
'''"""


CHATGLM3_PROMPT_TFQ = PromptTemplate(input_variables=["context"],template=chatglm3_prompt_tfq)
CHATGLM3_PROMPT_MCQ = PromptTemplate(input_variables=["context"],template=chatglm3_prompt_mcq)
QIANFAN_PROMPT_TFQ = PromptTemplate(input_variables=["context"],template=qianfan_prompt_tfq)
QIANFAN_PROMPT_MCQ = PromptTemplate(input_variables=["context"],template=qianfan_prompt_mcq)


async def async_chat_mcq(input_text, model,num_selector):
    # yield gr.DataFrame(pd.DataFrame(), col_count=(4, "fixed"), row_count=(3, "fixed")) ,""
    global qianfanchain_mcq
    global chatglm3chain_mcq
    # Create an asynchronous callback handler
    callback = AsyncIteratorCallbackHandler()
    # Define an asynchronous function to wrap another asynchronous function and signal completion or exceptions using an event
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn  # Wait for the provided asynchronous function to complete
        except Exception as e:
            # TODO: Handle exceptions - here, we simply print the exception information
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()  # Set the event to indicate completion

    # Create a task to perform message generation with ChatOpenAI and monitor the completion event of the callback handler
    if model == "ernie":
        task = asyncio.create_task(wrap_done(qianfanchain_mcq.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    else:
        task = asyncio.create_task(wrap_done(chatglm3chain_mcq.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    print("*"*20)
    # Iterate asynchronously to obtain tokens from the callback handler
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
        # yield out_text,dataframe,"",gr.Button(visible=False)
    await task  # Wait for the task to complete

async def async_chat_tfq(input_text, model,num_selector):
    # yield gr.DataFrame(pd.DataFrame(), col_count=(4, "fixed"), row_count=(3, "fixed")) ,""
    global qianfanchain_tfq
    global chatglm3chain_tfq
    # Create an asynchronous callback handler
    callback = AsyncIteratorCallbackHandler()
    # Define an asynchronous function to wrap another asynchronous function and signal completion or exceptions using an event
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn  # Wait for the provided asynchronous function to complete
        except Exception as e:
            # TODO: Handle exceptions - here, we simply print the exception information
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()  # Set the event to indicate completion

    # Create a task to perform message generation with ChatOpenAI and monitor the completion event of the callback handler
    if model == "ernie":
        task = asyncio.create_task(wrap_done(qianfanchain_tfq.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    else:
        task = asyncio.create_task(wrap_done(chatglm3chain_tfq.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    print("*"*20)
    # Iterate asynchronously to obtain tokens from the callback handler
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    await task  # Wait for the task to complete


def on_select(evt: gr.SelectData, df):
    print(f"You selected {evt.value} at {evt.index} from {evt.target}")
    if evt.value == "删除":
        df.drop(df.index[evt.index[0]],axis=0,inplace=True)
        return gr.DataFrame(df,interactive=True,column_widths=["30%", "60%","10%"])
    else:
        return df

def parse_mcq(output_text):
    questions = []
    question_texts = output_text.split('\n\n')
    for question_text in question_texts:
        lines = question_text.split('\n')
        question = lines[1]
        options = lines[2:-1]
        correct_answer = lines[-1].split("：")[1]
        options = [option.replace(". ",".") for option in options]
        question_dict = {
            'question': question.split('：')[1],
            'options': [option for option in options],
            'correct_answer': correct_answer,
        }
        questions.append(question_dict)
    lenth = len(questions)
    df = pd.DataFrame({
        "问题": [question['question'] for question in questions],
        "选项": [question['options'] for question in questions],
        "正确答案": [question['correct_answer'] for question in questions],
        "删除": ["删除" for _ in questions],
    })
    return gr.DataFrame(df, row_count=(lenth, "fixed"))

def parse_tfq(output_text):
    questions = []
    question_texts = output_text.strip().split('\n\n')
    for question_text in question_texts:
        lines = question_text.split('\n')
        question = lines[1].split('：')[1]
        correct_answer = lines[2].split('：')[1]
        explanation = lines[-1].split('：')[1]
        question_dict = {
            'question': question,
            'correct_answer': correct_answer,
            'explanation': explanation
        }
        questions.append(question_dict)
    lenth = len(questions)
    df = pd.DataFrame({
        "问题": [question['question'] for question in questions],
        "答案": [question['correct_answer'] for question in questions],
        "解析": [question['explanation'] for question in questions],
        "删除": ["删除" for _ in questions]
    })
    return gr.DataFrame(df, row_count=(lenth, "fixed"))


with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">辅助生成培训资料</h1>""")
    # with gr.Row():
    # input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10)
    with gr.Row():
        input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9)
        model_selector = gr.Dropdown(choices=["ernie","chatglm3"], label="请选择一个模型", scale=1, min_width=50, value="chatglm3")
    with gr.Row():
        num_selector = gr.Slider(minimum=0, maximum=10, value=5, label="请选择问题数量",step=1)
    with gr.Row():
        mcqBtn = gr.Button("生成选择题培训资料")
        tfqBtn = gr.Button("生成判断题培训资料")
    
    dataframe = gr.DataFrame(visible=True,interactive=True,column_widths=["30%", "40%", "20%", "10%"],col_count=(4, "fixed"), row_count=(1, "fixed"))
    dataframe.select(on_select, inputs=[dataframe], outputs=[dataframe])
    
    gr.Markdown("""---""")
    output_text = gr.Textbox(show_label=True, placeholder="输出...", lines=10)

    # clearBtn = gr.Button("清除")
    # clearBtn.click(clear, [], [dataframe, output_text])
    mcqBtn.click(
                 async_chat_mcq, [input_text, model_selector,num_selector], [output_text], queue=True
                ).then(
                 parse_mcq,[output_text], [dataframe], queue=False
                )
    tfqBtn.click(
                 async_chat_tfq, [input_text, model_selector,num_selector], [output_text], queue=True
                ).then(
                 parse_tfq,[output_text], [dataframe], queue=False
                )
    
    
if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=7658)
    parese.add_argument("--host", type=str, default="192.168.0.66")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.22.106:8003")
    
    args = parese.parse_args()
    global base_llm_url, qianfanchain_mcq, qianfanchain_tfq, chatglm3chain_mcq, chatglm3chain_tfq
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    base_llm1=ChatGLMSerLLM(url=base_llm_url)
    base_llm2=ChatERNIESerLLM(chat_completion=ChatCompletion(ak="pT7sV1smp4AeDl0LjyZuHBV9", sk="b3N0ibo1IKTLZlSs7weZc8jdR0oHjyMu"))
    qianfanchain_mcq = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_MCQ,llm_kwargs={"temperature":0.9})
    qianfanchain_tfq = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_TFQ,llm_kwargs={"temperature":0.9})
    chatglm3chain_mcq = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_MCQ,llm_kwargs={"temperature":0.9})
    chatglm3chain_tfq = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_TFQ,llm_kwargs={"temperature":0.9})
    
    
    demo.queue().launch(share=False, inbrowser=True,server_name=args.host,server_port=args.port)
