# -*- coding: utf-8 -*-
import os, sys

import pandas as pd
sys.path.append("../..") 

import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate

from typing import Awaitable
import asyncio
from langchain.callbacks  import AsyncIteratorCallbackHandler
import re
from llm.ernie_with_sdk import ChatERNIESerLLM
from qianfan import ChatCompletion
chatglm3_prompt_qa = """请根据以下资料内容生成问答对，一共输出{num_selector}个问题。（请你按照以下格式进行回答）
Q：问题
A：答案
Q：问题
A：答案
...
资料内容如下所示：
{context}"""

qianfan_prompt_qa = """请根据以下资料内容生成问答对，一共输出{num_selector}个问题。（请你按照以下格式进行回答）
Q：问题
A：答案
Q：问题
A：答案
...
资料内容如下所示：
'''
{context}
'''"""


CHATGLM3_PROMPT_QA = PromptTemplate(input_variables=["context"],template=chatglm3_prompt_qa)
QIANFAN_PROMPT_QA = PromptTemplate(input_variables=["context"],template=qianfan_prompt_qa)

async def async_chat_qa(input_text, model,num_selector):
    # yield gr.DataFrame(pd.DataFrame(), col_count=(3, "fixed"), row_count=(3, "fixed")) ,""
    global qianfanchain_qa
    global chatglm3chain_qa

    # Create an asynchronous callback handler
    callback = AsyncIteratorCallbackHandler()
    # Define an asynchronous function to wrap another asynchronous function and signal completion or exceptions using an event
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn  # Wait for the provided asynchronous function to complete
        except Exception as e:
            # TODO: Handle exceptions - here, we simply print the exception information
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()  # Set the event to indicate completion

    # Create a task to perform message generation with ChatOpenAI and monitor the completion event of the callback handler
    if model == "ernie":
        task = asyncio.create_task(wrap_done(qianfanchain_qa.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    else:
        task = asyncio.create_task(wrap_done(chatglm3chain_qa.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))

    print("*"*20)
    # Iterate asynchronously to obtain tokens from the callback handler
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    await task  # Wait for the task to complete

def on_select(evt: gr.SelectData, df):  # SelectData is a subclass of EventData
    print(f"You selected {evt.value} at {evt.index} from {evt.target}")
    # 删除选择行
    if evt.index[1] == 2:
        df.drop(df.index[evt.index[0]],axis=0,inplace=True)
        return gr.DataFrame(df,interactive=True)
    else:
        return df
    
def parse_qa(output_text):
    output_text = output_text.replace("\n\n", "\n")
    output_text += "\n"
    qa_pairs = re.findall(r"Q：(.*?)A：(.*?)\n", output_text, re.DOTALL)
    lenth = len(qa_pairs)
    # formatted_qa_pairs = [(q,a) for q, a in qa_pairs]
    df = pd.DataFrame({
        "Q": [q for q, _ in qa_pairs],
        "A": [a for _, a in qa_pairs],
        "删除": ["删除" for _, _ in qa_pairs]
    })
    return gr.DataFrame(df, row_count=(lenth, "fixed"))


with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">辅助生成知识库</h1>""")
    # with gr.Row():
    # input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10)
    with gr.Row():
        input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9)
        model_selector = gr.Dropdown(choices=["ernie","chatglm3"], label="请选择一个模型", scale=1, min_width=50, value="chatglm3")
    with gr.Row():
        num_selector = gr.Slider(minimum=0, maximum=10, value=5, label="请选择问题数量",step=1)
    with gr.Row():
        qaBtn = gr.Button("QA问答对生成")
    
    dataframe = gr.DataFrame(visible=True,interactive=True,column_widths=["30%", "60%", "10%"], col_count=(3, "fixed"), row_count=(1, "fixed"))
    dataframe.select(on_select, inputs=[dataframe], outputs=[dataframe])
    
    gr.Markdown("""---""")
    output_text = gr.Textbox(show_label=True, placeholder="输出...", lines=10)

    # clearBtn = gr.Button("清除")
    # clearBtn.click(clear, [], [dataframe, output_text])
    qaBtn.click(async_chat_qa, [input_text, model_selector,num_selector], [ output_text], queue=True).then(
            parse_qa,
            [output_text],
            [dataframe],
            queue=False
    )
    
    
if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=7658)
    parese.add_argument("--host", type=str, default="192.168.0.66")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.22.106:8003")
    
    args = parese.parse_args()
    global base_llm_url, qianfanchain_qa, chatglm3chain_qa
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    base_llm1=ChatGLMSerLLM(url=base_llm_url)
    base_llm2=ChatERNIESerLLM(chat_completion=ChatCompletion(ak="pT7sV1smp4AeDl0LjyZuHBV9", sk="b3N0ibo1IKTLZlSs7weZc8jdR0oHjyMu"))
    qianfanchain_qa = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_QA,llm_kwargs={"temperature":0.9})
    chatglm3chain_qa = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_QA,llm_kwargs={"temperature":0.9})
    
    
    demo.queue().launch(share=False, inbrowser=True,server_name=args.host,server_port=args.port)
