# -*- coding: utf-8 -*-
import os, sys
sys.path.append("../..") 

import pandas as pd
import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate

from typing import Awaitable
import asyncio
from langchain.callbacks  import AsyncIteratorCallbackHandler
import re
import json
from llm.ernie_with_sdk import ChatERNIESerLLM
from qianfan import ChatCompletion
# chatglm3_prompt_struct = """<|system|>
# 你是一个可以将一段文本根据其内容划分出多个副标题（千万不要推理出原文中不存在的内容），并按照固定的JSON格式输出的工具。JSON格式如下所示：
# ```{{
#     "副标题1": 字符串，第一个副标题,
#     "内容1": 一个字符串，生成副标题1的原文内容,
#     "副标题2": 一个字符串，显示根据原文内容生成的主题,
#     "内容2": 一个字符串，生成副标题2的原文内容,
#     "副标题3": 一个字符串，显示根据原文内容生成的主题,
#     "内容3": 一个字符串，生成副标题3的原文内容,
#     ...
# }}```
# <|user|>
# 文本内容如下：
# {context}"""

# qianfan_prompt_struct = """现有一段文本，请你根据其内容划分出多个副标题（不要推理出原文中不存在的内容），并按照固定的JSON格式输出的工具。JSON格式如下所示(注意每个内容结尾的“,”，保证JSON可以被正常解析)：
# ```{{
#     "副标题1": 字符串，第一个副标题,
#     "内容1": 字符串，生成副标题1的原文内容,
#     "副标题2": 字符串，显示根据原文内容生成的主题,
#     "内容2": 字符串，生成副标题2的原文内容,
#     "副标题3": 字符串，显示根据原文内容生成的主题,
#     "内容3": 字符串，生成副标题3的原文内容,
#     ...
# }}```

# 文本内容如下：
# '''
# {context}
# '''"""

chatglm3_prompt_struct_s1 = """<|system|>
你是一个可以将一段文本根据其内容重新划分其段落并按照一定格式进行输出的工具。你需要按照如下的格式进行输出：
段落1：
段落内容：

段落2：
段落内容：

...
-------------------------
注意：必须保证所有的段落内容之和为文本原文
-------------------------
<|user|>
文本内容如下：
{context}"""

chatglm3_prompt_struct_s2 = """<|system|>
你是一个可以为多个段落提取标题并按照一定格式进行输出的工具。你需要按照如下的格式进行输出：
段落1：
标题：（段落标题）
段落内容：（原文内容）

段落1：
标题：（段落标题）
段落内容：（原文内容）

...
<|user|>
文本内容如下：
{context}"""

qianfan_prompt_struct_s1 = """请根据以下材料的的内容对其进行段落划分，并按照以下格式进行输出：
段落1：
段落内容：

段落2：
段落内容：

...
（注意：必须保证所有的段落内容之和为文本原文，不要进行总结和推理）
文本内容如下：
```
{context}
```"""

qianfan_prompt_struct_s2 = """请为下面的每一段文字各取一个文章标题，并按照以下格式进行输出：
段落1：
标题：（段落标题）
段落内容：（原文内容）

段落1：
标题：（段落标题）
段落内容：（原文内容）

...
（注意：段落内容必须与原始文本内容保持一致）
文本内容如下：
```
{context}
```"""

CHATGLM3_PROMPT_STRUCT_S1 = PromptTemplate(input_variables=["context"],template=chatglm3_prompt_struct_s1)
CHATGLM3_PROMPT_STRUCT_S2 = PromptTemplate(input_variables=["context"],template=chatglm3_prompt_struct_s2)
# QIANFAN_PROMPT_STRUCT = PromptTemplate(input_variables=["context"], template=qianfan_prompt_struct)
QIANFAN_PROMPT_STRUCT_S1 = PromptTemplate(input_variables=["context"], template=qianfan_prompt_struct_s1)
QIANFAN_PROMPT_STRUCT_S2 = PromptTemplate(input_variables=["context"], template=qianfan_prompt_struct_s2)

def chat(input_text):
    global llmchain
    if not input_text:
        return ""
    # tags_box = gr.Dropdown.update(choices=["1","2","3"], value=["1","2"])
    result = llmchain.run({"context":input_text})
    return result

async def async_chat_stc(input_text, model):
    global base_llm1, base_llm2, step
    if step%2 == 1:
        qianfanchain_stc = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STRUCT_S1,llm_kwargs={"temperature":0.9})
        chatglm3chain_stc = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STRUCT_S1,llm_kwargs={"temperature":0.9})
        step = step + 1
    else:
        qianfanchain_stc = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STRUCT_S2,llm_kwargs={"temperature":0.9})
        chatglm3chain_stc = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STRUCT_S2,llm_kwargs={"temperature":0.9})
        step = step + 1
    # Create an asynchronous callback handler
    callback = AsyncIteratorCallbackHandler()
    # Define an asynchronous function to wrap another asynchronous function and signal completion or exceptions using an event
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn  # Wait for the provided asynchronous function to complete
        except Exception as e:
            # TODO: Handle exceptions - here, we simply print the exception information
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()  # Set the event to indicate completion

    # Create a task to perform message generation with ChatOpenAI and monitor the completion event of the callback handler
    if model == "ernie":
        task = asyncio.create_task(wrap_done(qianfanchain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))
    else:
        task = asyncio.create_task(wrap_done(chatglm3chain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))

    print("*"*20)
    # Iterate asynchronously to obtain tokens from the callback handler
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    await task  # Wait for the task to complete

# def get_jsonstr(output_test):
#     if output_test[0] == '{' and output_test[-1] == '}':
#         return output_test
#     pattern1 = r'```json(.*?)```'
#     match1 = re.search(pattern1, output_test, re.DOTALL)

#     if match1:
#         return match1.group(1)
#     else:
#         pattern2 = r'```(.*?)```'
#         match2 = re.search(pattern2, output_test, re.DOTALL)
#         if match2:
#             return match2.group(1)
#         else:
#             pattern3 = r'{(.*?)}'
#             match3 = re.search(pattern3, output_test, re.DOTALL)
#             if match3:
#                 return '{'+match3.group(1)+'}'
#             else:
#                 return None

def on_select(evt: gr.SelectData, df):  # SelectData is a subclass of EventData
    print(f"You selected {evt.value} at {evt.index} from {evt.target}")
    # 删除选择行
    if evt.index[1] == 2:
        df.drop(df.index[evt.index[0]],axis=0,inplace=True)
        return gr.DataFrame(df,interactive=True,column_widths=["30%", "60%","10%"])
    else:
        return df

# def parse_stc(output_text):
#     output_text = output_text.replace("\"\n    \"", "\",\n    \"")
#     print(output_text)
#     jsonstr = get_jsonstr(output_text)
#     numlines = int((len(jsonstr.splitlines()) - 2) / 2)
#     data = {}
#     try:
#         data = json.loads(jsonstr)
#     except Exception as e:
#         print(data)
#         print(f"在大模型给的结果转化为json形式的时候出现错误: {e}")
#     titles = []
#     texts = []
#     delete = []
#     for i in range(numlines):
#         title = data["副标题"+str(i+1)] 
#         titles.append(title)
#         text = data["内容"+str(i+1)]
#         texts.append(text)
#         delete.append("删除")
#     df = pd.DataFrame({
#         "标题": titles,
#         "内容": texts,
#         "删除": delete
#     })
#     return gr.DataFrame(df, row_count=(numlines, "fixed"))
    
def parse_stc(output_text):
    paragraphs = re.split(r'段落\d+：\n', output_text)[1:]
    titles = []
    contents = []
    delete = []
    for p in paragraphs:
        p = p.replace("段落内容：\n", "段落内容：")
        p = p.replace("标题：\n", "标题：")
        title_match = re.search(r'标题：(.+?)\n', p)
        content_match = re.search(r'段落内容：(.+)', p, re.DOTALL)
        if title_match and content_match:
            titles.append(title_match.group(1))
            contents.append(content_match.group(1))
            delete.append("删除")
    df = pd.DataFrame({"标题": titles, "段落内容": contents, "删除": delete})
    print(df)
    return gr.DataFrame(df, row_count=(len(titles), "fixed"))

with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">文章结构拆分</h1>""")
    # with gr.Row():
    with gr.Row():
        input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9)
        model_selector = gr.Dropdown(choices=["ernie","chatglm3"], label="请选择一个模型", scale=1, min_width=50, value="chatglm3")
    submitBtn = gr.Button("文本结构化")
    dataframe = gr.DataFrame(visible=True,interactive=True,column_widths=["30%", "60%","10%"],col_count=(3, "fixed"), row_count=(1, "fixed"))
    dataframe.select(on_select,inputs=[dataframe],outputs=[dataframe])

    gr.Markdown("""---""")
    output_text1 = gr.Textbox(show_label=True, placeholder="输出第一步...", lines=10, label="第一步输出")
    output_text2 = gr.Textbox(show_label=True, placeholder="输出第二步...", lines=10, label="第二步输出")
    submitBtn.click(
                 async_chat_stc, [input_text, model_selector], [output_text1], queue=True
                ).then(
                 async_chat_stc, [output_text1, model_selector], [output_text2], queue=True
                ).then(
                 parse_stc,[output_text2], [dataframe], queue=False
                )
    
    
if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=7653)
    parese.add_argument("--host", type=str, default="192.168.0.66")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.22.106:8003")
    
    global step
    step = 1
    args = parese.parse_args()
    global base_llm_url,llmchain,base_llm1,base_llm2
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    base_llm1=ChatGLMSerLLM(url=base_llm_url)
    base_llm2=ChatERNIESerLLM(chat_completion=ChatCompletion(ak="pT7sV1smp4AeDl0LjyZuHBV9", sk="b3N0ibo1IKTLZlSs7weZc8jdR0oHjyMu"))
    # llmchain = LLMChain(llm=base_llm, prompt=CHATGLM3_PROMPT_STRUCT,llm_kwargs={"temperature":0.9})
    # openai = OpenAI(model_name="chatglm3-6b", openai_api_key="token1",openai_api_base=base_llm_url + "/v1")
    # llmchain = LLMChain(llm=openai, prompt=CHATGLM_PROMPT_CT,verbose=True,llm_kwargs={"temperature":1.0})
    
    demo.queue().launch(share=False, inbrowser=True, server_name=args.host, server_port=args.port)
