# -*- coding: utf-8 -*-
import os, sys
sys.path.append("../..") 

import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate

from typing import Awaitable
import asyncio
from langchain.callbacks  import AsyncIteratorCallbackHandler
import re
from llm.ernie_with_sdk import ChatERNIESerLLM
from qianfan import ChatCompletion
from difflib import Differ

chatglm3_prompt_typos = """请仔细阅读以下资料，更正资料中的错别字，并按照以下格式进行输出。
更正后的资料：
文本内容

文本内容：
{context}"""

qianfan_prompt_typos = """'''
{context}
'''

已知有上述文本，现需要你仔细阅读上述资料，更正资料中的错别字之后进行输出。你需要按照如下格式进行输出：
更正之后的资料：
"""

CHATGLM3_PROMPT_TYPOS = PromptTemplate(input_variables=["context"],template=chatglm3_prompt_typos)
QIANFAN_PROMPT_TYPOS = PromptTemplate(input_variables=["context"], template=qianfan_prompt_typos)

async def async_chat_typos(input_text, model):
    # yield gr.DataFrame(col_count=(3, "fixed")) ,""
    global base_llm1, base_llm2, step
    qianfanchain_stc = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_TYPOS,llm_kwargs={"temperature":0.9})
    chatglm3chain_stc = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_TYPOS,llm_kwargs={"temperature":0.9})
    # Create an asynchronous callback handler
    callback = AsyncIteratorCallbackHandler()
    # Define an asynchronous function to wrap another asynchronous function and signal completion or exceptions using an event
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn  # Wait for the provided asynchronous function to complete
        except Exception as e:
            # TODO: Handle exceptions - here, we simply print the exception information
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()  # Set the event to indicate completion

    # Create a task to perform message generation with ChatOpenAI and monitor the completion event of the callback handler
    if model == "ernie":
        task = asyncio.create_task(wrap_done(qianfanchain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))
    else:
        task = asyncio.create_task(wrap_done(chatglm3chain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))

    print("*"*20)
    # Iterate asynchronously to obtain tokens from the callback handler
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    await task  # Wait for the task to complete

def parse_typos(output_text):
    print(output_text)
    output_text = output_text.replace("\n\n", "\n")
    output_text = '\n'.join(output_text.splitlines()[1:])
    return output_text

def diff_texts(text1, text2):
    d = Differ()
    return [
        (token[2:], token[0] if token[0] != " " else None)
        for token in d.compare(text1, text2)
    ]

with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">文章纠错</h1>""")
    # with gr.Row():
    with gr.Row():
        input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9)
        model_selector = gr.Dropdown(choices=["ernie","chatglm3"], label="请选择一个模型", scale=1, min_width=50, value="chatglm3")
    submitBtn = gr.Button("提交")
    output_text = gr.Textbox(show_label=True, placeholder="输出...", lines=10)
    gr.Interface(
        diff_texts,
        [input_text, output_text],
        gr.HighlightedText(
            label="Diff",
            combine_adjacent=True,
            show_legend=True,
            color_map={"+": "red", "-": "green"}),
        theme=gr.themes.Base()
    )
    
    submitBtn.click(
                    async_chat_typos, [input_text, model_selector],[output_text], queue=True
                   ).then(
                    parse_typos, [output_text], [output_text], queue=False
                   )
    
    
    
if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=7655)
    parese.add_argument("--host", type=str, default="192.168.0.66")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.22.106:8003")
    
    args = parese.parse_args()
    global base_llm_url,llmchain,step
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    base_llm1=ChatGLMSerLLM(url=base_llm_url)
    base_llm2=ChatERNIESerLLM(chat_completion=ChatCompletion(ak="pT7sV1smp4AeDl0LjyZuHBV9", sk="b3N0ibo1IKTLZlSs7weZc8jdR0oHjyMu"))
    # llmchain = LLMChain(llm=base_llm, prompt=CHATGLM3_PROMPT_TYPOS,llm_kwargs={"temperature":0.9})
    # openai = OpenAI(model_name="chatglm3-6b", openai_api_key="token1",openai_api_base=base_llm_url + "/v1")
    # llmchain = LLMChain(llm=openai, prompt=CHATGLM_PROMPT_CT,verbose=True,llm_kwargs={"temperature":1.0})
    
    demo.queue().launch(share=False, inbrowser=True,server_name=args.host,server_port=args.port)
