import sys
sys.path.append("../..") 
from llm.chatglm import ChatGLMSerLLM
from langchain import LLMChain
import json
from langchain.prompts import StringPromptTemplate,PromptTemplate
import re


chatglm_prompt = """请仔细阅读以下文本内容，并根据文本的主题和关键信息，生成一个包含摘要、标签列表和分类信息的JSON结构。确保摘要准确捕捉文本的核心内容，标签列表精确反映文本的主旨，并且分类信息与文本内容紧密相关。请按照以下JSON结构格式输出结果：（注意，你的回复中只能有json）
```{{
"title": 给文本添加标题，标题简洁明了地概括文本的主题。,
"summary": 这里是文本的摘要，简洁明了地概括文本的主要内容。,
"tags": 为文本添加标签，结果为list。,
"category": 给文本添加分类，结果为字符串。
}}```
文本内容如下所示：
{context}
"""
CHATGLM_PROMPT = PromptTemplate(input_variables=["context"],template=chatglm_prompt)

chatglm_prompt_ct = """请仔细阅读以下文本内容，并根据文本的主题和关键信息，生成一个包含摘要、标签列表和分类信息的JSON结构。确保摘要准确捕捉文本的核心内容，标签列表精确反映文本的主旨，并且分类信息与文本内容紧密相关。请按照以下JSON结构格式输出结果：（注意，你的回复中只能有json）
```{{
"title": 给文本添加标题，标题简洁明了地概括文本的主题。,
"summary": 这里是文本的摘要，概括文本的主要内容。,
"tags": 提取文本中的关键词，结果为list。,
"category": 给文本添加分类，结果为字符串。
}}```
注意：category必须属于集合之一：[{category}]
------------------------
文本内容如下所示：
{context}"""

CHATGLM_PROMPT_CT = PromptTemplate(input_variables=["context", "category"],template=chatglm_prompt_ct)

query = """为什么说现代经济是信用经济？
    答：（1）现代经济运作的特点。信用关系无处不在；信用规模呈现不断扩张趋势；信用结构日趋复杂化 。
   （2）从信用关系的各部门分析：盈余与赤字、债权与债务。
   （3）从信用关系中的主体来分析。由于经济中广泛存在着专门调剂资金余缺的金融机构,借贷双方不需要直接见面,通过金融机构作为中介人,便可解决资金的融通,从而进一步促进了信用和信用关系的发展。信用的发展,又大大促进了生产力和经济的发展。"""


def extract_first_code_block(input_string):
    if input_string[0] == '{' and input_string[-1] == '}':
        return input_string
    pattern1 = r'```json(.*?)```'
    match1 = re.search(pattern1, input_string, re.DOTALL)

    if match1:
        return match1.group(1)
    else:
        pattern2 = r'```(.*?)```'
        match2 = re.search(pattern2, input_string, re.DOTALL)
        if match2:
            return match2.group(1)
        else:
            return None

def prase(answer):
    json_data = extract_first_code_block(answer) # 从输出中拿json数据
    # print("==================================================")
    # print(json_data)
    # print("==================================================")
    data = {}
    try:
        data = json.loads(json_data)
    except Exception as e:
        print(f"在大模型给的结果转化为json形式的时候出现错误: {e}")

    title = data.get("title", "")
    summary = data.get("summary", "")
    tags = data.get("tags", "")
    category = data.get("category", "")

    return title, summary, tags, category


def modelcall_prase(llmchain,input):
    result = llmchain.invoke({"context":input})
    title, summary, tags, category = prase(result["text"])
    return result, title, summary, tags, category


def modelcall_prase_ct(llmchain, input, category):
    result = llmchain.invoke({"context":input, "category":category})
    title, summary, tags, category = prase(result["text"])
    return result, title, summary, tags, category

chatglm_prompt_tags = """请认真阅读下段文本内容，并根据其内容生成一个或多个标签。你需要回复一个列表，该数组中的元素为与文本内容相符号的标签，除此之外无需回复其他内容。文本内容如下：
{context}"""

chatglm_prompt_title = """请认真阅读下段文本内容，并根据其内容信息生成其标题名称。你只需要给出标题，无需输出其他内容。文本内容如下：
{context}"""

chatglm_prompt_summary = """请认真阅读下段文本内容，并用一句话描述其中心内容。你只需要输出该句话，无需输出其他内容。文本内容如下：
{context}"""

chatglm_prompt_category = """请认真阅读下段文本内容，并在预定义好的类别中匹配与文本所示内容最相似的一个。你只需要输出该类别，无需输出其他内容。注意：必须是给出的类别中的一个，如果找不到与文本所述相匹配的类型则输出“不限”即可。文本内容如下：
{context}
预定义的类别如下：
{category}"""

CHATGLM_PROMPT_TAGS = PromptTemplate(input_variables=["context"],template=chatglm_prompt_tags)
CHATGLM_PROMPT_TITLE = PromptTemplate(input_variables=["context"],template=chatglm_prompt_title)
CHATGLM_PROMPT_SUMMARY = PromptTemplate(input_variables=["context"],template=chatglm_prompt_summary)
CHATGLM_PROMPT_CATEGORY = PromptTemplate(input_variables=["context", "category"],template=chatglm_prompt_category)

if __name__ == "__main__":
    base_llm=ChatGLMSerLLM(url="http://192.168.10.93:8000")
    # chose_llm = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT_TAGS)
    # tags_get1 = chose_llm.run({"context":query})
    # print(tags_get1)
    # chose_llm = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT_CATEGORY)
    # print(chose_llm.run({"context":query}))
    # chose_llm = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT_TITLE)
    # print(chose_llm.run({"context":query}))
    # chose_llm = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT_SUMMARY)
    # print(chose_llm.run({"context":query, "category":"热点问题、手机银行、转账汇款、账户管理、个人网银、信贷业务、自助设备"}))
    chose_llm = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT_CT)
    result, title, summary, tags, category, subcategories = modelcall_prase_ct(chose_llm,input = query,category = "金融，财政，计算机，学生，教师，学校，社会")
    print("result:",result)
    print("title",title)
    print("summary",summary)
    print("tags",tags)
    print("category",category)
    print("subcategories",subcategories)


# """{
#     "summary": "储蓄国债（电子式）和储蓄国债（凭证式）的特点和操作说明。",
#     "tags": [
#         "储蓄国债",
#         "电子式",
#         "凭证式",
#         "付息",
#         "兑付",
#         "提前兑取",
#         "非交易过户",
#         "财产证明",
#         "质押贷款"
#     ],
#     "category": "金融产品",
#     "subcategories": [
#         "债券",
#         "国债",
#         "投资"
#     ]
# }"""