


from langchain.vectorstores.faiss import FAISS

from langchain.embeddings.base import Embeddings
from langchain.embeddings.huggingface import (
    HuggingFaceEmbeddings,
)
import math
from langchain_core.documents import Document

def build_and_save_vectorfaiss(documents,folder_path: str, index_name: str = "psbc_tags",embedding: str = "C:\\Users\\15663\\AI\\models\\bge-large-zh-v1.5"):
    embedding = HuggingFaceEmbeddings(model_name=embedding)
    faiss_vectorstore = FAISS.from_documents(documents=documents,embedding=embedding)
    faiss_vectorstore.save_local(folder_path=folder_path,index_name=index_name)
    
    
def load_vectorfaiss(folder_path: str, index_name: str = "index",embedding: str = "C:\\Users\\15663\\AI\\models\\bge-large-zh-v1.5"):
    embedding = HuggingFaceEmbeddings(model_name=embedding)
    return FAISS.load_local(folder_path=folder_path,embeddings=embedding,index_name=index_name)

def search_tags(native_tags: list, faiss_vectorstore: FAISS, threshold: float=0.7):
    score_threshold = (1-threshold) * math.sqrt(2)
    tags = []
    advice_tags = []
    for native_tag in native_tags:
        res = faiss_vectorstore.similarity_search_with_score(query=native_tag, score_threshold=score_threshold, k=1)
        if len(res) == 0:
            advice_tags.append(native_tag)
        else:
            tags.append(res[0][0].page_content)
    return tags, advice_tags

if __name__ == "__main__":
    tags = ["个人存款账户实名制","金融机构","身份证件","实名证件","规定施行","国家外汇管理局","行政许可","实施办法","外汇管理","政务服务","电子化办理","听证","监督检查"]
    docs = [Document(page_content=tag) for tag in tags]
    
    build_and_save_vectorfaiss(documents=docs,folder_path="./vectorstore",index_name="psbc_tags")
    
    faiss_vectorstore = load_vectorfaiss(folder_path="./vectorstore",index_name="psbc_tags")
    
    threshold = 0.7
    result = search_tags(["金融机构"], faiss_vectorstore, threshold=threshold)
    print(result)
    
    