# -*- coding: utf-8 -*-
import os, sys

import pandas as pd
sys.path.append("../..") 

import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate

from typing import Awaitable
import asyncio
from langchain.callbacks  import AsyncIteratorCallbackHandler
import re
from llm.ernie_with_sdk import ChatERNIESerLLM
from qianfan import ChatCompletion
from difflib import Differ
from scenarios.psbc.generate_centens import split_plus
from scenarios.psbc.prompts import (
    CHATGLM3_PROMPT_QA,
    CHATGLM3_PROMPT_MCQ,
    CHATGLM3_PROMPT_STRUCT_S2,
    CHATGLM3_PROMPT_TFQ,
    CHATGLM3_PROMPT_TYPOS1,
    CHATGLM3_PROMPT_TYPOS2,
    CHATGLM3_PROMPT_STC1,
    CHATGLM3_PROMPT_STC2,
    QIANFAN_PROMPT_MCQ,
    QIANFAN_PROMPT_QA,
    QIANFAN_PROMPT_STRUCT_S2,
    QIANFAN_PROMPT_TFQ,
    QIANFAN_PROMPT_TYPOS1,
    QIANFAN_PROMPT_TYPOS2,
    QIANFAN_PROMPT_STC1,
    QIANFAN_PROMPT_STC2
)
from ac_pattern import sensitive_word_detection
from langchain.embeddings.huggingface import (
    HuggingFaceEmbeddings,
)
from sklearn.metrics.pairwise import cosine_similarity
import numpy as np
import copy

embedding_path = "C:\\Users\\15663\\AI\\models\\bge-large-zh-v1.5"
embedding = HuggingFaceEmbeddings(model_name=embedding_path)



async def async_chat_mcq(input_text, model, num_selector):
    callback = AsyncIteratorCallbackHandler()
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn
        except Exception as e:
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()
    if model == "ernie":
        qianfanchain_mcq = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_MCQ, llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(qianfanchain_mcq.arun({"context":input_text, "num_selector":num_selector}, callbacks=[callback]), callback.done))
    else:
        chatglm3chain_mcq = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_MCQ, llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(chatglm3chain_mcq.arun({"context":input_text, "num_selector":num_selector}, callbacks=[callback]), callback.done))
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    with open('output.txt', 'a', encoding="utf-8") as f:
        sys.stdout = f
        print("==================================================async_chat_mcq=====================================================")
        print(text)
        print("==================================================async_chat_mcq=====================================================")
        sys.stdout = sys.__stdout__
    await task

async def async_chat_tfq(input_text, model,num_selector):
    callback = AsyncIteratorCallbackHandler()
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn
        except Exception as e:
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()
    if model == "ernie":
        qianfanchain_tfq = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_TFQ,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(qianfanchain_tfq.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    else:
        chatglm3chain_tfq = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_TFQ,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(chatglm3chain_tfq.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    with open('output.txt', 'a', encoding="utf-8") as f:
        sys.stdout = f
        print("==================================================async_chat_tfq=====================================================")
        print(text)
        print("==================================================async_chat_tfq=====================================================")
        sys.stdout = sys.__stdout__
    await task

async def async_chat_qa(input_text, model,num_selector):
    callback = AsyncIteratorCallbackHandler()
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn
        except Exception as e:
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()
    if model == "ernie":
        qianfanchain_qa = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_QA,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(qianfanchain_qa.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))
    else:
        chatglm3chain_qa = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_QA,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(chatglm3chain_qa.arun({"context":input_text,"num_selector":num_selector},callbacks=[callback]),callback.done))

    print("async_chat_qa")
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"
    with open('output.txt', 'a', encoding="utf-8") as f:
        sys.stdout = f
        print("==================================================async_chat_qa======================================================")
        print(text)
        print("==================================================async_chat_qa======================================================")
        sys.stdout = sys.__stdout__
    await task

# async def async_chat_stc(input_text, model):
#     # input_text = input_text.replace("\n\n", "xx8x88x").replace("\n","").replace("xx8x88x","\n\n")
#     print(input_text)
#     global base_llm1, base_llm2
#     callback = AsyncIteratorCallbackHandler()
#     async def wrap_done(fn: Awaitable, event: asyncio.Event):
#         try:
#             await fn
#         except Exception as e:
#             import traceback
#             traceback.print_exc()
#             print(f"Caught exception: {e}")
#         finally:
#             event.set()
#     if model == "ernie":
#         qianfanchain_stc = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STRUCT_S2,llm_kwargs={"temperature":0.9})
#         task = asyncio.create_task(wrap_done(qianfanchain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))
#     else:
#         chatglm3chain_stc = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STRUCT_S2,llm_kwargs={"temperature":0.9})
#         task = asyncio.create_task(wrap_done(chatglm3chain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))
#     print("async_chat_stc")
#     text=""
#     async for token in callback.aiter():
#         text=text+token
#         yield f"{text}"
#     with open('output.txt', 'a', encoding="utf-8") as f:
#         sys.stdout = f
#         print("=================================================async_chat_stc======================================================")
#         print(text)
#         print("=================================================async_chat_stc======================================================")
#         sys.stdout = sys.__stdout__
#     await task

async def async_chat_typos(input_text, model, tone, switch_tone):
    global base_llm1, base_llm2, xls_file
    # print("-----------------tone_switch-----------------", switch_tone)
    # 敏感词检测
    input_text = sensitive_word_detection(xls_file,input_text)
    callback = AsyncIteratorCallbackHandler()
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn
        except Exception as e:
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()
    if model == "ernie" and switch_tone != False:
        qianfanchain_stc = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_TYPOS1,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(qianfanchain_stc.arun({"context":input_text, "tone":tone},callbacks=[callback]),callback.done))
    elif model != "ernie" and switch_tone != False:
        chatglm3chain_stc = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_TYPOS1,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(chatglm3chain_stc.arun({"context":input_text, "tone":tone},callbacks=[callback]),callback.done))
    elif model == "ernie" and switch_tone == False:
        qianfanchain_stc = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_TYPOS2,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(qianfanchain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))
    elif model != "ernie" and switch_tone == False:
        chatglm3chain_stc = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_TYPOS2,llm_kwargs={"temperature":0.9})
        task = asyncio.create_task(wrap_done(chatglm3chain_stc.arun({"context":input_text},callbacks=[callback]),callback.done))
    print("async_chat_typos")
    text=""
    async for token in callback.aiter():
        if text.startswith("更正之后的资料："):
            text = text[8:]
        if token.startswith("\n") and text.endswith("\n"):
            token = token[1:]
        text += token.replace("\n\n","\n").replace("\n","  \n")
        yield f"{text}"
    with open('output.txt', 'a', encoding="utf-8") as f:
        sys.stdout = f
        print("=================================================async_chat_typos====================================================")
        print(text)
        print("=================================================async_chat_typos====================================================")
        sys.stdout = sys.__stdout__
    await task

def on_select(evt: gr.SelectData, df):
    print(f"You selected {evt.value} at {evt.index} from {evt.target}")
    lenth = len(df)
    if evt.value == "删除":
        df.drop(df.index[evt.index[0]],axis=0,inplace=True)
        return gr.DataFrame(df,interactive=True, row_count=(lenth-1, "fixed"))
    else:
        return df

def parse_mcq(output_text):
    questions = []
    try:
        question_texts = output_text.split('\n\n')
        for question_text in question_texts:
            lines = question_text.split('\n')
            question = lines[1]
            options = lines[2:-1]
            temp = lines[-1].split("：")
            if len(temp) > 1 and temp[1] != "":
                correct_answer = temp[1]
            else:
                continue
            options = [option.replace(". ",".") for option in options]
            question_dict = {
                'question': question.split('：')[1],
                'options': [option for option in options],
                'correct_answer': correct_answer,
            }
            questions.append(question_dict)
        lenth = len(questions)
        df = pd.DataFrame({
            "问题": [question['question'] for question in questions],
            "选项": [question['options'] for question in questions],
            "正确答案": [question['correct_answer'] for question in questions],
            "删除": ["删除" for _ in questions],
        })
        return gr.DataFrame(df, row_count=(lenth, "fixed"))
    except:
        with open('output.txt', 'a', encoding="utf-8") as f:
            sys.stdout = f
            print("==================================================parse_mcq======================================================")
            print("模型格式输出超出预期，选择题解析出现错误！！！")
            print("==================================================parse_mcq======================================================")
            sys.stdout = sys.__stdout__
        return ""


def parse_tfq(output_text):
    questions = []
    try:
        question_texts = output_text.strip().split('\n\n')
        for question_text in question_texts:
            lines = question_text.split('\n')
            question = lines[1].split('：')[1]
            correct_answer = lines[2].split('：')[1]
            explanation = lines[-1].split('：')[1]
            question_dict = {
                'question': question,
                'correct_answer': correct_answer,
                'explanation': explanation
            }
            questions.append(question_dict)
        lenth = len(questions)
        df = pd.DataFrame({
            "问题": [question['question'] for question in questions],
            "答案": [question['correct_answer'] for question in questions],
            "解析": [question['explanation'] for question in questions],
            "删除": ["删除" for _ in questions]
        })
        return gr.DataFrame(df, row_count=(lenth, "fixed"))
    except:
        with open('output.txt', 'a', encoding="utf-8") as f:
            sys.stdout = f
            print("==================================================parse_tfq======================================================")
            print("模型输出格式超出预期，判断题解析出现错误！！！")
            print("==================================================parse_tfq======================================================")
            sys.stdout = sys.__stdout__
        return ""

def parse_qa(output_text):
    try:
        output_text = output_text.replace("\n\n", "\n")
        output_text += "\n"
        qa_pairs = re.findall(r"Q[0-9]*：(.*?)\nA[0-9]*：(.*?)\n", output_text, re.DOTALL)
        lenth = len(qa_pairs)
        df = pd.DataFrame({
            "Q": [q.replace("\n", "\\n") for q, _ in qa_pairs],
            "A": [a.replace("\n", "\\n") for _, a in qa_pairs],
            "删除": ["删除" for _, _ in qa_pairs]
        })
        return gr.DataFrame(df, row_count=(lenth, "fixed"))
    except:
        with open('output.txt', 'a', encoding="utf-8") as f:
            sys.stdout = f
            print("==================================================parse_qa=======================================================")
            print("模型输出格式超出预期，QA解析出现错误！！！")
            print("==================================================parse_qa=======================================================")
            sys.stdout = sys.__stdout__
        return ""

# def parse_stc(output_text):
#     paragraphs = re.split(r'段落\d+：\n', output_text)[1:]
#     titles = []
#     contents = []
#     delete = []
#     for p in paragraphs:
#         p = p.replace("段落内容：\n", "段落内容：")
#         p = p.replace("标题：\n", "标题：")
#         title_match = re.search(r'标题：(.+?)\n', p)
#         content_match = re.search(r'段落内容：(.+)', p, re.DOTALL)
#         if title_match and content_match:
#             titles.append(title_match.group(1))
#             contents.append(content_match.group(1).replace("\n", "\\n"))
#             delete.append("删除")
#     df = pd.DataFrame({"标题": titles, "段落内容": contents, "删除": delete})
#     print(df)
#     return gr.DataFrame(df, row_count=(len(titles), "fixed"))

# def parse_stc_text(output_text):
#     output_text = output_text.replace("\n\n", "\n")
#     output_text = re.sub(r'段落\d+：\n', '', output_text)
#     output_text = output_text.replace("标题：", "\n")
#     output_text = output_text.replace("段落内容：", "")
#     if output_text.splitlines()[0] == "":
#         output_text = '\n'.join(output_text.splitlines()[1:])
#     return output_text

def parse_stc_text(output_text1, output_text2):
    try:
        # combined_data = ""
        markdown_text = ""
        for paragraph1, paragraph2 in zip(output_text1.split("\n-------------new_para-------------\n"), output_text2.split("\n\n")):
            # paragraph1 = re.sub(r'段落\d+：\n段落内容：', '', paragraph1)
            # paragraph1 = re.sub(r'段落\d+：\n', '', paragraph1)
            # paragraph2 = re.sub(r'段落\d+：\n标题：', '', paragraph2)
            if paragraph2[-1] != "\n":
                paragraph2 = paragraph2 + "\n"
            # combined_data += paragraph2 + "" + paragraph1 + "\n\n"
            markdown_text += "### "+paragraph2 + "  \n&ensp;&ensp;&ensp;&ensp;" + paragraph1.replace("\n","  \n&ensp;&ensp;&ensp;&ensp;") + "  \n\n"
        return markdown_text
    except:
        with open('output.txt', 'a', encoding="utf-8") as f:
            sys.stdout = f
            print("==================================================parse_stc======================================================")
            print("模型输出格式超出预期，文本结构化出现错误！！！")
            print("==================================================parse_stc======================================================")
            sys.stdout = sys.__stdout__
        return ""

def parse_typos(output_text):
    try:
        # output_text = output_text.replace("  \n  \n", "  \n")
        if output_text.splitlines()[0] == "更正之后的资料：  " or output_text.splitlines()[0] == "更正之后的资料  ":
            output_text = '\n'.join(output_text.splitlines()[1:])
        # print([output_text])
        return output_text
    except:
        with open('output.txt', 'a', encoding="utf-8") as f:
            sys.stdout = f
            print("==================================================parse_typos====================================================")
            print("模型输出格式超出预期，错别字修改出现错误！！！")
            print("==================================================parse_typos====================================================")
            sys.stdout = sys.__stdout__
        return ""

def diff_texts(text1, text2):
    text2 = text2.replace("&ensp;", "")
    d = Differ()
    return [
        (token[2:], token[0] if token[0] != " " else None)
        for token in d.compare(text1, text2)
    ]

def save2file_mcq(dataframe):
    try:
        dataframe.to_csv("files/mcq.csv", mode='a', index=False)
        gr.Info("导出数据成功")
        print("导出数据成功")
    except:
        raise gr.Error("导出数据出错")
    
def save2file_tfq(dataframe):
    try:
        dataframe.to_csv("files/tfq.csv", mode='a', index=False)
        gr.Info("导出数据成功")
        print("导出数据成功")
    except:
        raise gr.Error("导出数据出错")

def save2file_qa(dataframe):
    try:
        dataframe.to_csv("files/qa.csv", mode='a', index=False)
        gr.Info("导出数据成功")
        print("导出数据成功")
    except:
        raise gr.Error("导出数据出错")    

def save2file_stc(dataframe):
    try:
        dataframe.to_csv("files/stc.csv", mode='a', index=False)
        gr.Info("导出数据成功")
        print("导出数据成功")
    except:
        raise gr.Error("导出数据出错")
    

def reset_dataframe_mcq_tfq():
    df = pd.DataFrame({
        "1": [],
        "2": [],
        "3": [],
        "4": []
    })
    return gr.DataFrame(df, visible=True,interactive=True,col_count=(4, "fixed"), row_count=(1, "fixed"))

def reset_dataframe_qa():
    df = pd.DataFrame({
        "1": [],
        "2": [],
        "3": []
    })
    return gr.DataFrame(df, visible=True, interactive=True, col_count=(3, "fixed"), row_count=(1, "fixed"))

def reset_highlighttext():
    return None

def reset_textbox_stc():
    return ""

def disable_btn_1():
    return gr.Button(interactive=False)

def disable_btn_2():
    return (gr.Button(interactive=False),)*2

def disable_btn_4():
    return (gr.Button(interactive=False),)*4

def enable_btn_1():
    return gr.Button(interactive=True)

def enable_btn_2():
    return (gr.Button(interactive=True),)*2

def enable_btn_3():
    return (gr.Button(interactive=True),)*3

def set_textbox_container_false():
    return gr.Textbox(container=False)

def set_textbox_container_true():
    return gr.Textbox(container=True)

def chat_stc(input_text, model):
    global llmchaim, base_llm1, base_llm2
    if model == "ernie":
        llmchain = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STRUCT_S2,llm_kwargs={"temperature":0.9})
    else:
        llmchain = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STRUCT_S2,llm_kwargs={"temperature":0.9})
    if not input_text:
        return ""
    # tags_box = gr.Dropdown.update(choices=["1","2","3"], value=["1","2"])
    result = llmchain.invoke({"context":input_text})
    with open('output.txt', 'a', encoding="utf-8") as f:
        sys.stdout = f
        print("====================================================chat_stc=========================================================")
        print(result["text"])
        print("====================================================chat_stc=========================================================")
        sys.stdout = sys.__stdout__
    return result["text"].replace("《","").replace("》","").replace("*","")

def stc_step1(input_text):
    split_threshold=0.8
    split_threshold_para=0.75
    _, para_split_simple = split_plus(input_text,split_threshold,split_threshold_para,marge_threshold_para=0.5)
    return "\n-------------new_para-------------\n".join(para_split_simple)

# 将该函数伪装成异步函数
async def stc_step2(output_text, model):
    output_text_splits = output_text.split("\n-------------new_para-------------\n")
    # print(output_text_splits)
    outans = ""
    for output_text_split in output_text_splits:
        outans = outans + chat_stc(output_text_split, model) + "\n\n"
    return outans

def detailed_description_of_tone(radio_input):
    """正式","友好","轻松","强调","客观"""
    if radio_input == "正式":
        return "使用规范的语法和词汇，结构严谨，用词严肃。常在正式场所使用"
    elif radio_input == "友好":
        return "使文本显得亲切和易于接受。常用于日常对话、商务交流、社交媒体等场合"
    elif radio_input == "轻松":
        return "语气轻松随意，是口语化的表达，包含较多缩写词、幽默等元素。常用于不太正式的场合"
    elif radio_input == "强调":
        return "强调某个观点或情感。常用演讲、辩论、广告等"
    elif radio_input == "客观":
        return "使用客观的词语和描述，并且尽可能避免主观因素的影响。常用于新闻报道、学术论文等"
    
def f_switch_tone(switch_tone):
    if switch_tone == False:
        return gr.Radio(interactive=False,value='')
    else:
        return gr.Radio(interactive=True)


def get_para_list(input_text):
    o_para_list = input_text.split("\n")
    para_list = []
    for index,item in enumerate(o_para_list):
        if len(item.strip()) == 0:
            if len(para_list) == 0:
                para_list.append(item+'\n')
            else:
                para_list[-1] = para_list[-1] + '\n' + item
        elif len(item) <= 20:
            if index+1 < len(o_para_list):
                o_para_list[index+1] = item+"\n"+o_para_list[index+1]
                continue
            else:
                para_list.append(item)
        else:
            para_list.append(item)
    print(para_list)
    if len(para_list) >= 2 and all(char.isspace() for char in para_list[0]):
        tmp_str = para_list[0]
        para_list = para_list[1:]
        para_list[0] = tmp_str+para_list[0]
    return para_list

class TitleInfo:
    def __init__(self):
        # 结构化信息
        # 当前标题在原始句子的位置信息，如第一个元素为5，则表示0-4为当前标题下的内容
        self.para_list = []
        # 详细相似信息，用于插入展示调试
        self.para_info = []
        # 当前标题下总文本数量，不包括插入的相似信息，因此父级标题展示offset位置错误 [{"offset":108,"similarity":[0.555,0.9]}]
        self.para_len = []
        # 当前级别标题，由模型生成
        self.title = []
        # 段落向量
        self.para_vec = []
        # 递归结构
        self.title_info = None
    
    def gen_title2(self, o_para_list, model_selector):
        global llmchain, base_llm1, base_llm2
        if self.title_info is not None:
            self.title_info.gen_title2(o_para_list, model_selector)
        if self.title_info is None:
            text = ""
            for index, para in enumerate(o_para_list):
                if index in self.para_list:
                    if model_selector == 'ernie':
                        llmchain = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STC1, llm_kwargs={"temperature":0.9})
                    elif model_selector == 'chatglm3':
                        llmchain = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STC1, llm_kwargs={"temperature":0.9})
                    result = llmchain.invoke({"context":text})
                    self.title.append(result["text"].replace("》", "").replace("《", "").replace("*", ""))
                    text = ""
                text = text+para+'\n'
            if model_selector == 'ernie':
                llmchain = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STC1, llm_kwargs={"temperature":0.9})
            elif model_selector == 'chatglm3':
                llmchain = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STC1, llm_kwargs={"temperature":0.9})
            result = llmchain.invoke({"context":text})
            self.title.append(result["text"].replace("》", "").replace("《", "").replace("*", ""))
        else:
            text = ""
            title = ""
            syg = -1
            iindex = 0
            positions = [self.title_info.para_list.index(num) for num in self.para_list]
            for index, para in enumerate(o_para_list):
                text = text+para+'\n'
                if index+1 in self.title_info.para_list:
                    ttitle_n = self.title_info.para_list.index(index+1)
                    title = title+self.title_info.title[ttitle_n]+'\n'
                if index+1 in self.para_list:
                    distance = positions[iindex] - syg
                    syg = positions[iindex]
                    iindex = iindex+1
                    if distance >= 4:
                        if model_selector == 'ernie':
                            llmchain = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STC2, llm_kwargs={"temperature":0.9})
                        elif model_selector == 'chatglm3':
                            llmchain = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STC2, llm_kwargs={"temperature":0.9})
                        result = llmchain.invoke({"context":title})
                        self.title.append(result["text"].replace("》", "").replace("《", "").replace("*", ""))
                        text = ""
                        title = ""
                    else:
                        if model_selector == 'ernie':
                            llmchain = LLMChain(llm=base_llm2, prompt=QIANFAN_PROMPT_STC1, llm_kwargs={"temperature":0.9})
                        elif model_selector == 'chatglm3':
                            llmchain = LLMChain(llm=base_llm1, prompt=CHATGLM3_PROMPT_STC1, llm_kwargs={"temperature":0.9})
                        result = llmchain.invoke({"context":text})
                        self.title.append(result["text"].replace("》", "").replace("《", "").replace("*", ""))
                        text = ""
                        title = ""
                    




    def gen_title1(self,para_list,llmchain,start=0,end=-1,para_len=-1):
        if end == -1:
            end = len(para_list)
        for i,index in enumerate(self.para_list):
            if index > end:
                break
            if index <= start:
                continue
            if self.title_info is not None:
                self.title_info.gen_title1(para_list,llmchain,start=start,end=index,para_len=self.para_len[i])
            
            if self.para_len[i] < para_len or para_len == -1:       # 当标题下的文本长度相同是，不在展示标题
                text = "\n".join(para_list[start:index])
                result = llmchain.invoke({"context":text})
                self.title.append(result["text"].replace("》", "").replace("《", "").replace("*", ""))

            else:
                self.title.append("")
            start = index



    def show(self,para_list,start=0,end=-1,para_len=-1,parent_no="",level=""):
        if end == -1:
            end = len(para_list)
        result=""
        pa = 0
        for i,index in enumerate(self.para_list):
            title_no = parent_no+str(pa+1)+"."
            if index > end:
                break
            if index <= start:
                continue
            if self.para_len[i] < para_len or para_len == -1:       # 当标题下的文本长度相同是，不在展示标题
                pa += 1
                if len(self.title) > i:
                    result+= level+"# "+title_no+"&ensp;"+self.title[i]+"\n"
                else:
                    result+= level+"# "+title_no+"&ensp;此处为标题"+str(index)+"\n"   # 使用self.title 替换
            if self.title_info is not None:
                result+= self.title_info.show(para_list,start=start,end=index,para_len=self.para_len[i],parent_no=title_no,level=level+"#")
            else:
                # pa += 1
                result+= "&ensp;&ensp;&ensp;&ensp;"+"  \n&ensp;&ensp;&ensp;&ensp;".join(para_list[start:index])
                result+="  \n"
            start = index
        return result
    def show_detail(self,para_list,start=0,end=-1,para_len=-1,parent_no="",level=""):
        if end == -1:
            end = len(para_list)
        result=""
        pa = 0
        for i,index in enumerate(self.para_list):
            title_no = parent_no+str(pa+1)+"."
            if index > end:
                break
            if index <= start:
                continue
            if self.para_len[i] < para_len or para_len == -1: 
                pa += 1
                if len(self.title) > i:
                    result+= level+"# "+title_no+"&ensp;"+self.title[i]+"\n"
                else:
                    result+= level+"# "+title_no+"&ensp;此处为标题"+str(index)+"\n"   # 使用self.title 替换
            if self.title_info is not None:
                # 打印高级标题相似度
                for k,para_i in enumerate(self.para_info[i]):
                    result +='**{}与{}相似{:.5}** 与上句相似{:.5}||'.format(self.title_info.para_list[k],self.title_info.para_list[k+1],para_i["similarity"][0],para_i["similarity"][1])
                result+="  \n"
                result+= self.title_info.show_detail(para_list,start=start,end=index,para_len=self.para_len[i],parent_no=title_no,level=level+"#")
            else:
                detail_para = "&ensp;&ensp;&ensp;&ensp;"+self.insert_info("\n".join(para_list[start:index]),self.para_info[i])
                detail_para = detail_para.replace("\n","  \n&ensp;&ensp;&ensp;&ensp;")
                result+= detail_para
                result+="  \n"
            start = index
        return result
    def is_root(self):
        if len(self.para_list) == 1:
            return True
        else:
            title_info = self.title_info
            level = 1
            while title_info is not None:
                title_info = title_info.title_info
                level += 1
                if level > 2:       # 达到四级标题
                    return True
            return False
    def insert_info(self,para,info):
        for i in range(len(info)-1,-1,-1):
            para = para[:info[i]["offset"]] + "**与上段落相似 {:.5} 与上句相似 {:.5}**\n".format(info[i]["similarity"][0],info[i]["similarity"][1]) +para[info[i]["offset"]:]
        return para


def spilt_plus(para_list,split_threshold,split_threshold_para,title_info:TitleInfo = None):
    if title_info is not None and title_info.is_root():
        return title_info
    title_info_parent = TitleInfo()
    # 通过title_info构建新的片段
    _para_list = []
    _para_list_index = []
    _para_vec = []
    if title_info is None or not title_info.para_info:
        _para_list = para_list
        _para_list_index = [i+1 for i in range(len(_para_list))]
        for index in _para_list:
            _para_vec.append(np.array(embedding.embed_query(index)).reshape(1,-1))
    else:
        offset = 0
        _para_vec = copy.deepcopy(title_info.para_vec)
        for index in title_info.para_list:
            _para_list_index.append(index)
            if index <= len(para_list) and index >= offset:
                _para_list.append("\n".join(para_list[offset:index]))
                offset = index
            else:
                print(f"段落信息错误，index:{index},offset:{offset}")
    cur_para = _para_list[0]
    cur_para_info = []
    cur_para_vec = _para_vec[0]
    last_centence_vec = cur_para_vec
    for index in range(1,len(_para_list)):
        vec = _para_vec[index]
        similarity = cosine_similarity(cur_para_vec,vec)
        similarity2 = cosine_similarity(last_centence_vec,vec)
        if similarity > split_threshold_para or similarity2 > split_threshold:
            cur_para_info.append({"offset":len(cur_para)+1,"similarity":[similarity[0][0],similarity2[0][0]]})
            cur_para += "\n" + _para_list[index]
            cur_para_vec = embedding.embed_query(cur_para)
            cur_para_vec = np.array(cur_para_vec).reshape(1,-1)
        else:
            title_info_parent.para_list.append(_para_list_index[index-1])
            title_info_parent.para_len.append(len(cur_para))
            title_info_parent.para_vec.append(cur_para_vec)
            title_info_parent.para_info.append(cur_para_info)
            cur_para = _para_list[index]
            cur_para_info=[{"offset":0,"similarity":[similarity[0][0],similarity2[0][0]]}]
            cur_para_vec = vec
        last_centence_vec = vec

    if len(cur_para)>0:
        title_info_parent.para_list.append(_para_list_index[len(_para_list)-1])
        title_info_parent.para_info.append(cur_para_info)
        title_info_parent.para_len.append(len(cur_para))
        title_info_parent.para_vec.append(cur_para_vec)
    title_info_parent.title_info = title_info
    return title_info_parent


def marge_single_title(para_list,title_info:TitleInfo, marge_threshold_para=0):
    if title_info is None or title_info.title_info is not None: # 如果不是叶子标题，不需要合并
        return title_info
    marge_title_info = TitleInfo()
    offset = 0
    cache_para_info = []
    cache_len = 0
    cache_vec = None
    # last_para_vec = None
    for index in range(len(title_info.para_list)):
        if len(title_info.para_info[index])>1:
            # 当前段落由多个原句组成，则不在合并
            # 处理向量，和标题下文本长度
            if cache_vec is not None:       # 如果存在合并缓存，重新计算向量
                text = "\n".join(para_list[offset:title_info.para_list[index]])
                marge_title_info.para_vec.append(np.array(embedding.embed_query(text)).reshape(1,-1))
                marge_title_info.para_len.append(len(text))
            else:
                marge_title_info.para_vec.append(title_info.para_vec[index])
                marge_title_info.para_len.append(title_info.para_len[index])
            # 处理文本位置信息
            marge_title_info.para_list.append(title_info.para_list[index])
            # 处理合并信息
            for info1 in title_info.para_info[index]:
                info1["offset"] += cache_len if cache_len ==0 else cache_len + 1
                cache_para_info.append(info1)
            marge_title_info.para_info.append(cache_para_info)
            # 初始化信息
            cache_para_info = []
            cache_len = 0
            offset = title_info.para_list[index]
            cache_vec = None
        else:
            next_similarity = -0.1
            if index < len(title_info.para_list)-1:
                # next_similarity = title_info.para_info[index+1][0]["similarity"][0]
                next_similarity = cosine_similarity(title_info.para_vec[index],title_info.para_vec[index+1])[0][0]
                next_similarity = max(next_similarity,title_info.para_info[index+1][0]["similarity"][0])
            last_similarity = -0.1
            if cache_vec is not None:       # 存在缓存
                last_similarity = cosine_similarity(title_info.para_vec[index],cache_vec)[0][0]
            elif len(marge_title_info.para_vec)>0:
                last_similarity = cosine_similarity(title_info.para_vec[index],marge_title_info.para_vec[-1])[0][0]
            if index > 0: # 第一段没有与上一段的相似度
                last_similarity = max(title_info.para_info[index][0]["similarity"][0],last_similarity)

            if last_similarity > marge_threshold_para or next_similarity > marge_threshold_para: # 需要合并
                if last_similarity > next_similarity and cache_vec is None: #合并到上一段
                    marge_title_info.para_list[-1] = title_info.para_list[index]
                    for info1 in title_info.para_info[index]:
                        info1["offset"] += marge_title_info.para_len[-1] + 1
                        marge_title_info.para_info[-1].append(info1)
                    marge_title_info.para_len[-1] += title_info.para_len[index]+1
                    marge_title_info.para_vec[-1] = np.array(embedding.embed_query(
                        "\n".join(para_list[offset:title_info.para_list[index]]))).reshape(1,-1)
                    # 初始化信息
                    cache_para_info = []
                    cache_len = 0
                    offset = title_info.para_list[index]
                    cache_vec = None
                else: # 将当前段落合并到cache中
                    text = "\n".join(para_list[offset:title_info.para_list[index]])
                    # 处理合并信息
                    for info1 in title_info.para_info[index]:
                        info1["offset"] += cache_len if cache_len ==0 else cache_len + 1
                        cache_para_info.append(info1)
                    cache_len = len(text)
                    cache_vec = np.array(embedding.embed_query(text)).reshape(1,-1)
                    # cache_para_info = title_info.para_info[index]
            else:
                # 当前段落不合并，则将cache中的内容合并到marge_title_info中
                if cache_vec is not None:       # 如果存在合并缓存，重新计算向量
                    marge_title_info.para_vec.append(cache_vec)
                    marge_title_info.para_len.append(cache_len)
                    marge_title_info.para_list.append(title_info.para_list[index-1])
                    marge_title_info.para_info.append(cache_para_info)
                marge_title_info.para_vec.append(title_info.para_vec[index])
                marge_title_info.para_len.append(title_info.para_len[index])
                marge_title_info.para_list.append(title_info.para_list[index])
                marge_title_info.para_info.append(title_info.para_info[index])
                # 初始化信息
                cache_para_info = []
                cache_len = 0
                offset = title_info.para_list[index]
                cache_vec = None
    if cache_vec is not None:       # 如果存在合并缓存，重新计算向量
        marge_title_info.para_vec.append(cache_vec)
        marge_title_info.para_len.append(cache_len)
        marge_title_info.para_list.append(title_info.para_list[-1])
        marge_title_info.para_info.append(cache_para_info)
    marge_title_info.title_info = title_info.title_info
    return marge_title_info


async def deal_para(para, model_selector):
    split_method = '合并细碎叶子段落'
    split_threshold = 0.8
    split_threshold_para = 0.75
    marge_threshold_para = 0.5
    parent_threshold = 0.9
    global base_llm, llmchain
    import time
    start = time.time()
    title_info = None
    para_list = get_para_list(para)
    # 防止过度循环
    level = 0
    while level < 4:
        title_info = spilt_plus(para_list,split_threshold,split_threshold_para,title_info)
        if title_info.title_info is None and split_method == "合并细碎叶子段落":
            # 试图合并段落
            title_info = marge_single_title(para_list,title_info,marge_threshold_para)
        if title_info.title_info is not None and len(title_info.para_list) == len(title_info.title_info.para_list):  # 如果标题没变，则说明没有找到合适的标题
            title_info = title_info.title_info
            break
        level += 1
        title_info2 = title_info
        while title_info2 is not None:
            print(title_info2.para_list)
            title_info2 = title_info2.title_info
        if title_info.is_root():
            break
        # 相似度按80%递减
        split_threshold *= parent_threshold
        split_threshold_para *= parent_threshold
    print("处理耗时:",time.time()-start)
    start = time.time()
    title_info.gen_title2(para_list, model_selector)
    print("处理耗时:",time.time()-start)
    with open('output.txt', 'a', encoding="utf-8") as f:
        sys.stdout = f
        print("=================================================async_stc_title====================================================")
        x = title_info
        while x:
            print(x.title)
            x=x.title_info
        print("=================================================async_stc_title====================================================")
        sys.stdout = sys.__stdout__
    return title_info.show(para_list)

with gr.Blocks(title="AI辅助知识库生成工具集") as demo:
    gr.HTML("""<h1 align="center">AI辅助知识库生成工具集</h1>""")
    model_selector = gr.Dropdown(choices=["ernie","chatglm3"], label="请选择一个模型", scale=1, min_width=50, value="ernie")
    with gr.Tab("辅助生成培训资料"):
        with gr.Row():
            input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9, label="文本输入")
        with gr.Row():
            num_selector = gr.Slider(minimum=1, maximum=7, value=5, label="请选择问题数量",step=1)
        with gr.Row():
            mcqBtn = gr.Button("生成选择题培训资料")
            tfqBtn = gr.Button("生成判断题培训资料")
        dataframe = gr.DataFrame(visible=True,interactive=True,col_count=(4, "fixed"), row_count=(1, "fixed"))
        dataframe.select(on_select, inputs=[dataframe], outputs=[dataframe])
        with gr.Row():
            storeBtn_mcq = gr.Button("选择题资料导出", interactive=False)
            storeBtn_tfq = gr.Button("判断题资料导出", interactive=False)
        gr.Markdown("""---""")
        output_text = gr.Textbox(placeholder="输出...", lines=10, show_label=True, label="模型输出")
        mcqBtn.click(
                        disable_btn_4, [], [mcqBtn, tfqBtn, storeBtn_mcq, storeBtn_tfq], queue=False
                    ).then(
                        reset_dataframe_mcq_tfq, [], [dataframe], queue=False
                    ).then(
                        set_textbox_container_false, [], [output_text], queue=False
                    ).then(
                        async_chat_mcq, [input_text, model_selector, num_selector], [output_text], queue=True
                    ).then(
                        set_textbox_container_true, [], [output_text], queue=False
                    ).then(
                        parse_mcq,[output_text], [dataframe], queue=False
                    ).then(
                        enable_btn_3, [], [mcqBtn, tfqBtn, storeBtn_mcq], queue=False
                    )
        tfqBtn.click(
                        disable_btn_4, [], [mcqBtn, tfqBtn, storeBtn_mcq, storeBtn_tfq], queue=False
                    ).then(
                        reset_dataframe_mcq_tfq, [], [dataframe], queue=False
                    ).then(
                        set_textbox_container_false, [], [output_text], queue=False
                    ).then(
                        async_chat_tfq, [input_text, model_selector,num_selector], [output_text], queue=True
                    ).then(
                        set_textbox_container_true, [], [output_text], queue=False
                    ).then(
                        parse_tfq,[output_text], [dataframe], queue=False
                    ).then(
                        enable_btn_3, [], [mcqBtn, tfqBtn, storeBtn_tfq], queue=False
                    )
        storeBtn_mcq.click(
                        save2file_mcq, [dataframe]
                    ).then(
                        disable_btn_2, [], [storeBtn_mcq, storeBtn_tfq], queue=False
                    )
        storeBtn_tfq.click(
                        save2file_tfq, [dataframe]
                    ).then(
                        disable_btn_2, [], [storeBtn_mcq, storeBtn_tfq], queue=False
                    )

    with gr.Tab("辅助生成知识库"):
        with gr.Row():
            input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9, label="文本输入")
        with gr.Row():
            num_selector = gr.Slider(minimum=1, maximum=7, value=5, label="请选择问题数量",step=1)
        with gr.Row():
            qaBtn = gr.Button("QA问答对生成")
        dataframe = gr.DataFrame(visible=True, interactive=True, col_count=(3, "fixed"), row_count=(1, "fixed"))
        dataframe.select(on_select, inputs=[dataframe], outputs=[dataframe])
        storeBtn = gr.Button("QA问答对导出", interactive=False)
        gr.Markdown("""---""")
        output_text = gr.Textbox(placeholder="输出...", lines=10, show_label=True, label="模型输出")
        qaBtn.click(
                    disable_btn_2, [], [qaBtn, storeBtn], queue=False
                ).then(
                    reset_dataframe_qa, [], [dataframe], queue=False
                ).then(
                    set_textbox_container_false, [], [output_text], queue=False
                ).then(
                    async_chat_qa, [input_text, model_selector,num_selector], [output_text], queue=True
                ).then(
                    set_textbox_container_true, [], [output_text], queue=False
                ).then(
                    parse_qa, [output_text], [dataframe], queue=False
                ).then(
                    enable_btn_2, [], [qaBtn, storeBtn], queue=True
                )
        storeBtn.click(
                    save2file_qa, [dataframe]
                ).then(
                    disable_btn_1, [], [storeBtn], queue=False 
                )

    with gr.Tab("文章结构拆分"):
        with gr.Row():
            input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, scale=9, label="文本输入")
        # split_method = gr.Radio(choices=["基于连续两句话相似度分割","基于已分割段落与句子相似度分割","基于已分割和连续同时判断","基于已分割和连续同时判断，并合并独立段落"], label="分割方式",value="基于连续两句话相似度分割")
        # with gr.Row():
        #     split_threshold = gr.Slider(0.5,1.0,label="句子相似度阈值", value=0.8,step=0.01)
        #     split_threshold_para = gr.Slider(0.5,1.0,label="段落相似度阈值", value=0.75,step=0.01)
        submitBtn = gr.Button("文本结构化")
        # dataframe = gr.DataFrame(visible=True, interactive=True, col_count=(3, "fixed"), row_count=(1, "fixed"))
        # dataframe.select(on_select,inputs=[dataframe],outputs=[dataframe])
        # storeBtn = gr.Button("文本结构化导出")
        # stc_text = gr.Textbox(show_label=True, placeholder="等待文本结构化结果...", lines=10, label="文本结构化结果展示：")
        gr.Markdown("""# 文本结构化展示""")
        gr.Markdown("""---""")
        markdown_text = gr.Markdown()
        
        submitBtn.click(
                        disable_btn_1, [], [submitBtn]
                    ).then(
                        reset_textbox_stc, [], [markdown_text]
                    ).then(
                        deal_para, [input_text, model_selector], [markdown_text]
                    ).then(
                        enable_btn_1, [], [submitBtn]
                    )
        # storeBtn.click(save2file_stc, [dataframe])
        
    with gr.Tab("文本修正优化"):
        with gr.Row():
            input_text = gr.Textbox(show_label=True, placeholder="输入需要处理的文档...", lines=10, label="文本输入")
            with gr.Column():
                gr.Markdown("""## 文章修正优化效果展示""")
                markdown_text = gr.Markdown()
        with gr.Row():
            switch_tone = gr.Checkbox(show_label=False, value=False, scale=1, label="文本风格", interactive=True, container=True)
            tone_selector = gr.Radio(show_label=False, min_width=500,choices=["正式","友好","轻松","强调","客观"], interactive=False, label="文本语气选择",value="", scale=4)
            tone_textbox = gr.Textbox(show_label=False, lines=1, scale=5, label="语气介绍", interactive=False, value="")
        submitBtn = gr.Button("文本修正优化")
        gr.Markdown("""---""")
        with gr.Row():
            diff = gr.HighlightedText(
                label="Diff",
                combine_adjacent=True,
                show_legend=True,
                color_map={"+": "red", "-": "#C0C0C0"})
        switch_tone.change(f_switch_tone, [switch_tone], [tone_selector])
        tone_selector.change(detailed_description_of_tone, [tone_selector], [tone_textbox])
        submitBtn.click(
                        disable_btn_1, [], [submitBtn], queue=False
                    ).then(
                        reset_highlighttext, [], [diff], queue=False
                    ).then(
                        async_chat_typos, [input_text, model_selector, tone_selector, switch_tone],[markdown_text], queue=True
                    ).then(
                        parse_typos, [markdown_text], [markdown_text], queue=False
                    ).then(
                        diff_texts, [input_text, markdown_text], [diff], queue=False
                    ).then(
                        enable_btn_1, [], [submitBtn], queue=False
                    )
        
if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=7654)
    parese.add_argument("--host", type=str, default="0.0.0.0")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.22.106:8003")
    parese.add_argument("--sensitive_word", type=str, default="敏感词.xls",help="敏感词记录文件")
    args = parese.parse_args()
    global base_llm_url,llmchain,base_llm1,base_llm2,xls_file
    xls_file = args.sensitive_word
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    base_llm1=ChatGLMSerLLM(url=base_llm_url)
    base_llm2=ChatERNIESerLLM(chat_completion=ChatCompletion(ak="pT7sV1smp4AeDl0LjyZuHBV9", sk="b3N0ibo1IKTLZlSs7weZc8jdR0oHjyMu"), model_name="ERNIE-Bot-4")
    demo.queue().launch(share=False, inbrowser=True, server_name=args.host, server_port=args.port)