# -*- coding: utf-8 -*-

import os, sys
sys.path.append("../..") 

import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain_openai import OpenAI
from langchain.prompts import PromptTemplate

from typing import Awaitable
import asyncio
from langchain.callbacks  import AsyncIteratorCallbackHandler
from llm.ernie_with_sdk import ChatERNIESerLLM
from qianfan import ChatCompletion
chatglm_prompt_ct = """请仔细阅读以下资料，更正资料中的错别字，并按照以下格式输出：
更正后的资料：
文本内容

文本内容：
{context}"""

chatglm_prompt_ct2 = """请仔细阅读以下资料，更正资料中语法错误、错别字，并按要求输出。
输出格式为json：
```json
[
    {{
        "ori_text": 需要修改的原始句子,
        "repare_text": 修改后的句子
    }}
]
------------------------
资料：
{context}"""

qianfan_prompt_ct1 = """请根据下面提供的知识资料，生成可以作为考试的判断题，并给出正确答案。按照如下格式进行回答：
试题1：试题内容。
正确答案：正确/错误。
解析：选择原因（如果用到原资料中的内容，请列出来）

试题2：试题内容。
正确答案：正确/错误。
解析：选择原因（如果用到原资料中的内容，请列出来）

...

资料内容如下所示：
'''
{context}
'''"""

qianfan_prompt_ct2 = """现有一段文本，请你根据其内容划分出多个副标题（不要推理出原文中不存在的内容），并按照固定的JSON格式输出的工具。JSON格式如下所示：
```{{
    "副标题1": 字符串，第一个副标题,
    "内容1": 字符串，生成副标题1的原文内容,
    "副标题2": 字符串，显示根据原文内容生成的主题,
    "内容2": 字符串，生成副标题2的原文内容,
    "副标题3": 字符串，显示根据原文内容生成的主题,
    "内容3": 字符串，生成副标题3的原文内容,
    ...
}}```
请你保证JSON格式的正确性，在输出之前确定格式可以被解析
文本内容如下：
'''
{context}
'''"""

qianfan_prompt_ct3 = """请根据以下资料内容生成问答对，（请你按照以下格式进行回答）
Q：问题
A：答案
Q：问题
A：答案
...
资料内容如下所示：
'''
{context}
'''"""

qianfan_prompt_ct4 = """请你根据下面这一段文本的内容生成可以用于考试的选择题，并按照一定格式进行输出的工具。具体的输出格式如下所示:
试题1：
题目：。
A.选项内容    B.选项内容    C.选项内容    D.选项内容
正确答案：给出具体选项。

试题2：
题目：。
A.选项内容    B.选项内容    C.选项内容    D.选项内容
正确答案：给出具体选项。
...

文本内容如下：
'''
{context}
'''"""
qianfan_prompt_ct5 = """请仔细阅读以下资料，更正资料中的错别字，并按照以下格式进行输出。
更正后的资料：
文本内容

文本内容：
'''
{context}
'''"""

qianfan_prompt_ct6 = """请根据以下材料的的内容对其进行结构划分，并按照以下格式进行输出（注意：你应保证所有段落的段落内容之和为文本原文）：
段落1：
段落内容：

段落2：
段落内容：

...

文本内容如下：
```
{context}
```"""

qianfan_prompt_ct7 = """请为下面的每一段文字各取一个文章标题，并按照以下格式进行输出：
段落1：
标题：
段落内容：

段落1：
标题：
段落内容：

...

文本内容如下：
```
{context}
```"""

qianfan_prompt_ct8 = """'''
{context}
'''

已知有上述文本，现需要你仔细阅读上述资料，更正资料中的错别字之后进行输出。你需要按照如下格式进行输出：
更正之后的资料：
"""

qianfan_prompt_ct9 = """请为下面的每一段文字各取一个文章标题，并按照以下格式进行输出：
段落1：
标题：
段落内容：

段落1：
标题：
段落内容：

...

文本内容如下：
```
{context}
```"""

CHATGLM_PROMPT_CT = PromptTemplate(input_variables=["context"],template=chatglm_prompt_ct)

async def async_chat(input_text):
    global llmchain
    # Create an asynchronous callback handler
    callback = AsyncIteratorCallbackHandler()
    # Define an asynchronous function to wrap another asynchronous function and signal completion or exceptions using an event
    async def wrap_done(fn: Awaitable, event: asyncio.Event):
        try:
            await fn  # Wait for the provided asynchronous function to complete
        except Exception as e:
            # TODO: Handle exceptions - here, we simply print the exception information
            import traceback
            traceback.print_exc()
            print(f"Caught exception: {e}")
        finally:
            event.set()  # Set the event to indicate completion

    # Create a task to perform message generation with ChatOpenAI and monitor the completion event of the callback handler
    task = asyncio.create_task(wrap_done(llmchain.arun({"context":input_text},callbacks=[callback]),callback.done))
    print("*"*20)
    # Iterate asynchronously to obtain tokens from the callback handler
    text=""
    async for token in callback.aiter():
        text=text+token
        yield f"{text}"  # Convert tokens to strings and yield them

    await task  # Wait for the task to complete
def chat(input_text):
    global llmchain
    if not input_text:
        return ""
    # tags_box = gr.Dropdown.update(choices=["1","2","3"], value=["1","2"])
    result = llmchain.run({"context":input_text})
    return result

    
def reset_state():
    return "",""

def change_prompt(text_prompt):
    global llmchain
    new_prompt = PromptTemplate(input_variables=["context"],template=text_prompt)
    llmchain.prompt = new_prompt
    
def reset_prompt():
    global llmchain
    llmchain.prompt = CHATGLM_PROMPT_CT
    return chatglm_prompt_ct
    
with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">文本归档</h1>""")
    text_prompt = gr.Textbox(show_label=False, value=chatglm_prompt_ct,placeholder="提示词...", lines=5)
    with gr.Row():
        promptBtn = gr.Button("提交")
        resetBtn = gr.Button("重置")
    with gr.Row():
        input_text = gr.Textbox(show_label=False, placeholder="输入需要处理的文档...", lines=10)
        output_text = gr.Textbox(show_label=False, placeholder="输出...", lines=10)
    with gr.Row():
        submitBtn = gr.Button("提交")
        emptyBtn = gr.Button("清除")

    submitBtn.click(async_chat, [input_text],
                    [output_text], show_progress=True)
    promptBtn.click(change_prompt,[text_prompt],[])
    resetBtn.click(reset_prompt,outputs=[text_prompt])
    emptyBtn.click(reset_state, outputs=[input_text,output_text])
    gr.Examples(label="文案举例",examples=[
["""2020 年，新冠肺炎疫情席卷全球，世界经济发展的不确定性大幅增加。各国政府、企业探索采用新兴技术降低经济社会运行成本，提升实体经济运行效率，进一步寻找经济发展新的增长点。当前，区块链与云计算、人工智能等新技术基础设施交叉创新，越来越多的实体经济垂直领域呈现出“区块链+”的发展格局和“脱虚向实”的良好势头。
区块链产业长期向好，行业认知回归理性。随着中央对区块链技术发展的规划指引，我国区块链明确了以联盟链为基础，围绕服务实体经济、优化公共服务为目标的发展思路，产业发展方向进一步清晰。
现阶段广大从业者对区块链的信心持续向好，普遍认可区块链的长期战略性价值。与此同时，政策制定方、技术提供方、系统使用方等各类行业参与者已认识到区块链的发展并非只差临门一脚，而是尚处于行业发展的初期，需要产业结合实际情况，务实探索区块链的应用落地路径，合力解决赋能实体经济过程中的问题和挑战。
区块链技术创新依旧活跃，工程化和生态构建成为重点。从技术层面看，区块链技术还在发展早期，专利申请、学术研究等方面保持活跃，但为了应用尽快落地，行业不再片面追求新技术创新，而是进入务实发展阶段。具体系统开发过程中，技术要求主要是好用、易用、安全、性能好、支持互操作，技术发展呈现出工程化导向。同时各类区块链产品之间的技术差异逐步缩小，技术提供方将更多精力投入到生态构建当中，降低开发部署门槛，提升用户粘性，吸纳更多的开发者、使用者，以此构建自己的生态壁垒。未来一段时间，区块链行业技术发展将主要聚焦于工程化和生态构建。
区块链应用存证先行，逐渐向自动化协作和价值互联迈进。当前区块链技术分布广泛，应用牵头方呈现出技术服务方推动，政府主导的态势。经过近年来的发展，区块链在供应链金融、溯源、公共服务等领域取得一定成果，但其应用模式仍以文件、合同等的存证为主。
其他行业受限于数字化程度不足、合法合规性等因素约束，应用发展相对缓慢。区块链针对实体经济的核心价值正是促进产业上下游高效协作，提升产融结合效能。因此，区块链产业应用逐步向政务数据共享、供应链协同、跨境贸易等自动化协作和价值互联迈进。
区块链基础设施化呼声渐起，建设模式仍需深度探索。区块链构建可信数据共享环境的价值已经获得多方认可，各机构对利用区块链进行可信存证的需求逐步显现，技术使用方渴望可以使用易用性强、标准化程度高的区块链通用性产品，对区块链基础设施化的呼声已经出现。各个国家、地区尝试建设服务地域内多个组织的区块链基础设施，其中主要代表有欧盟 EBSI、美洲开发银行 LACChain 等。此外，我国将多云跨云 BaaS 服务、开放联盟链等作为区块链基础设施的探索。可以看到，区块链基础设施将是未来长远发展方向，但具体建设模式仍将在前进中持续探索，不断演进。
区块链不能包打天下，需与多技术配合完成数字化转型。正如TCP/IP 协议是可靠通信的基础协议，区块链则是可信交易的基础组件，但是这并不意味着区块链就是解决行业所有痛点的“万能钥匙”。
区块链在下一代信息技术创新发展中具有不可替代的作用，作为行业赋能的工具，单纯使用区块链技术构建的应用也难以发挥其价值。这就需要区块链与人工智能、物联网、大数据、5G 等其他技术相结合，利用协同效应形成一体化解决方案，共同助力数字化转型。"""],
["""服务进出口是指除去商品进出口以外的那部分国际收支。获得收入可看作服务出口，支出则可当作服务进口：服务这种商品，也被看作是无形商品。国际贸易包括许多不同种类的服务。
(1)Tourism and Transportation (旅游业、交通运输业) 交通运输业和旅游业是国际航空公司、航运公司、预约服务机构和旅馆收益的主要来源。
就国家而言，像希腊和挪威这样的国家，收入的大部分要依赖航运业，巴哈马群岛的收入则更多的来自海外游客，而不是靠其商品出口。
(2) Performance of activities abroad (国外商务活动) 企业要为其在国外进行的某些活动支付服务费用，这些服务包括银行、保险、租赁、工程和管理等。工程服务常常是以交钥匙工程承包方式进行的。承建生产设施的合同规定工程完成时，将全部设施交付转让给物主。服务管理费用的支付以管理合同为依据，根据这一合同，其中某公司为对方公司提供管理人员进行一般管理或专门管理。
(3) Use of assets from abroad (国外资产的运用) 特许使用费是使用国外资产所支付的费用，如商标、专利、版权或其他类似许可证协定那样的合同项目下的专业技术。特许使用费也可以用于特许经营。在这种经营方式中， 一方当事人(给予特许者)将其商标的使用权出售给另一方独立的当事人(特许持有人),该使用权即成为特许权持有人的基本资产，此外，特许者还要给予特许持有人业务上的资助，如提供零配件，提供管理服务或技术等。
企业在成功地建立起出口市场之后，往往倾向于采用国外许可证交易或特许经营。这种转移往往比早期单纯出口要承担更多的国际义务。这是因为企业通常需要向国外派遣技术人员以帮助被许可人或特许持有人建立和选择新产品的生产设施。"""],
["""合资企业是根据合资各方同意的规章而建立的一种商业关系，它涉及资产的统筹，共同经营管理，分享利润和分担风险。从法律上讲，合资企业是一种合伙关系， 一种可以被各种产业合作形式所采取的商业机构模式；换句话说，就是以独立和联合的形式，把共同销售、服务、生产等方面按法律程序组织成一个合资企业。合资企业可以是股权式合营企业，也可以是非股权式合营企业。如果是前者，将建立一个独立的合资实体，当地合资者购买合资资本的部分股权。如果是后者，就不再建立另外的合资实体，联合完全建立在合同的基础上。
不论是股权式合营企业，还是非股权式合营企业都要共同承担风险，分享利润。合资企业比出口贸易、进口贸易或许可证贸易更加稳定，比独自经营的费用要低。然而，合资企业也面临一些问题，如控制问题，其原因是由于合资企业要求跨国合作，还有诸如所有权比例，投资数量以及出口量多少，如何公平分配利润等问题。"""]], inputs=[input_text])
    gr.Examples(label="提示词举例",examples=[[qianfan_prompt_ct1],[qianfan_prompt_ct2],[qianfan_prompt_ct3],[qianfan_prompt_ct4],[qianfan_prompt_ct5],[qianfan_prompt_ct6],[qianfan_prompt_ct7],[qianfan_prompt_ct8]], inputs=[text_prompt])
if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=8081)
    parese.add_argument("--host", type=str, default="192.168.0.66")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.0.148:8000")
    
    args = parese.parse_args()
    global base_llm_url,llmchain
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    base_llm=ChatERNIESerLLM(chat_completion=ChatCompletion(ak="pT7sV1smp4AeDl0LjyZuHBV9", sk="b3N0ibo1IKTLZlSs7weZc8jdR0oHjyMu"))
    llmchain = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT_CT,llm_kwargs={"temperature":0.9})
    # openai = OpenAI(model_name="chatglm3-6b", openai_api_key="token1",openai_api_base=base_llm_url + "/v1")
    # llmchain = LLMChain(llm=openai, prompt=CHATGLM_PROMPT_CT,verbose=True,llm_kwargs={"temperature":1.0})
    
    demo.queue().launch(share=False, inbrowser=True,server_name="0.0.0.0",server_port=args.port)






