
import os, sys
sys.path.append("../..") 

import gradio as gr
import argparse

from llm.chatglm import ChatGLMSerLLM
from langchain.chains import LLMChain
from langchain_openai import OpenAI
from langchain.prompts import StringPromptTemplate,PromptTemplate
from scenarios.psbc.model_serve import modelcall_prase,modelcall_prase_ct,CHATGLM_PROMPT,CHATGLM_PROMPT_CT
from langchain.vectorstores.faiss import FAISS
from langchain_core.documents import Document
from langchain.embeddings.huggingface import (
    HuggingFaceEmbeddings,
)
from scenarios.psbc.tag_memory_store.vectorstore import search_tags
tags = ["贷款业务","即期交易","贴现市场","远期外汇","买入汇率","外汇交易","货币政策","通货膨胀","浮动汇率","套算汇率",
        "票据贴现","外汇市场","即期汇率","汇率差价","短期票据","市场机制","期货交易","债务关系","免责条款","远期汇率",
        "套期保值","本票","国际信贷","保险单据","套汇策略","商业汇票","保值策略","市场风险","利息率期权","私募债券",
        "浮动利率","不可撤销信用证","固定汇率","可撤销信用证","固定利率","汇款业务","货币互换","票据行为","通货期权","利息率期货",
        "跟单汇票","票据权利","风险保值","跟单信用证","汇率风险","远期本票","货币贬值","布雷顿森林货币体系","保兑信用证","黄金市场",
        "资产负债比率","外汇保值条款","远期外汇交易","对外贸易保险","国际金融市场","承兑汇票","国际支付","国际结算","外汇证券市场","银行贷款",
        "信用证","自由交易市场","贸易规则","借款协议","保底期权","特别提款权","税收政策","贴现率","出口融资","信用形式",
        "票据法","国际债券","外汇资产","国际法律","资信调查","民间经贸组织","国际贸易术语","国际贸易结算规则","国际黄金市场","货币制度",
        "金币本位制","金块本位制","金本位制度","汇率机制","汇率变动","国际经济关系","利率计算","外汇保值","套汇活动","双边经济援助",
        "利率互换","期权交易","股票期权交易","票据清算制度","利率政策","贷款利率","商业发票","国际非贸易结算","信用凭证","标价法"]

category=["国际金融市场","跟单信用证与国际惯例","国际结算","国际贸易信用与结算","国际贸易"]

def chat(state,input_text,tags_box,filter_num):
    global llmchain,vectorstore
    if not input_text:
        return {},"","",[],[],gr.CheckboxGroup([],value=[]),"",""
    print(tags_box)
    # tags_box = gr.Dropdown.update(choices=["1","2","3"], value=["1","2"])
    # result = llmchain.run({"question":input_text})
    result, title, summary, tags_out, category_out, = modelcall_prase_ct(llmchain,input_text,category=",".join(category))
    # result, title, summary, tags_out, category_out, = modelcall_prase(llmchain,input_text)
    tags_filter,tags_suggest = search_tags(native_tags=tags_out,faiss_vectorstore=vectorstore,threshold=filter_num)
    if "text" in result:
        output_text = gr.Textbox(visible=True,value=result["text"])
    else:
        output_text = gr.Textbox(visible=True,value=result)
    state = {"tags_suggest":tags_suggest,"model_tags":tags_out}
    tags_box_suggest = gr.CheckboxGroup(tags_suggest,value=[])
    return state,title,category_out,tags_out,tags_filter,tags_box_suggest,summary,output_text

def adopt_tags(state,tags_box_suggest,tags_box_filter):
    if len(tags_box_suggest) <= 0:
        return state,tags_box_suggest,tags_box_filter
    tags_box_filter.extend(tags_box_suggest)
    tags_box_filter_update = gr.Dropdown(tags, value=tags_box_filter)

    new_suggest_tags = []
    for tag in state["tags_suggest"]:
        if tag not in tags_box_suggest:
            new_suggest_tags.append(tag)
    state["tags_suggest"] = new_suggest_tags
    tags_box_suggest_update = gr.CheckboxGroup(new_suggest_tags,value=[])
    
    return state,tags_box_suggest_update,tags_box_filter_update

def filter_change(state,filter_num,tags_box_suggest,tags_box_filter):
    if not state or "model_tags" not in state or len(state["model_tags"]) == 0:
        return state,tags_box_suggest,tags_box_filter
    global llmchain,vectorstore
    
    tags_filter,tags_suggest = search_tags(native_tags=state["model_tags"],faiss_vectorstore=vectorstore,threshold=filter_num)
    state["tags_suggest"] = tags_suggest
    tags_box_suggest_update = gr.CheckboxGroup(tags_suggest,value=[])
    return state,tags_box_suggest_update,tags_filter
    
    
def reset_state():
    output_text = gr.Textbox(visible=False,value="")
    return {},"","","",[],[],gr.CheckboxGroup([],value=[]),"",output_text
def dropdown_event_handler(value):
    print("Dropdown value selected: ", value)
    
with gr.Blocks(title="AI辅助工具") as demo:
    gr.HTML("""<h1 align="center">文本归档</h1>""")
    input_text = gr.Textbox(show_label=False, placeholder="输入需要处理的文档...", lines=10)
    with gr.Row():
        submitBtn = gr.Button("提交")
        emptyBtn = gr.Button("清除")

    title_box = gr.Textbox(label="标题", placeholder="标题", interactive=True)
    with gr.Row():
        category_box = gr.Dropdown(category,label="类别",  allow_custom_value=True,interactive=True,scale=1,min_width=300)
        with gr.Column(scale=3):
            tags_box_filter = gr.Dropdown(tags, value=[], multiselect=True, label="标签",interactive=True,allow_custom_value=True)
            tags_box_suggest = gr.CheckboxGroup([],value=[],interactive=True, label="建议标签")
        with gr.Column(scale=1):
            filter_num = gr.Slider(minimum=0.5, maximum=1.0, value=0.85, step=0.01, interactive=True, label="标签匹配度",min_width=300)
            adoptBtn = gr.Button("采纳")
                    
                # tags_box_suggest = gr.Dropdown([], value=[], multiselect=True, label="建议标签",interactive=True,allow_custom_value=True)
    summary_box = gr.Textbox(label="摘要", placeholder="摘要", interactive=True)
    state = gr.State()
    ## 插入多个空行
    gr.Markdown("""---""")
    with gr.Column():
        ## 圆角矩形框，显示文字并有删除按钮
        tags_box = gr.Dropdown([], value=[], multiselect=True, label="标签",interactive=True,allow_custom_value=True,scale=6)
        output_text = gr.Textbox(label="输出",visible=False, placeholder="Output...", lines=10)

    submitBtn.click(chat, [state,input_text,tags_box,filter_num],
                    [state,title_box, category_box, tags_box,tags_box_filter,tags_box_suggest,summary_box,output_text], show_progress=True)
    adoptBtn.click(adopt_tags,[state,tags_box_suggest,tags_box_filter],[state,tags_box_suggest,tags_box_filter])

    filter_num.change(filter_change,[state,filter_num,tags_box_suggest,tags_box_filter],[state,tags_box_suggest,tags_box_filter])
    
    emptyBtn.click(reset_state, outputs=[state,input_text, title_box, category_box, tags_box,tags_box_filter,tags_box_suggest,summary_box,output_text])
    gr.Examples(examples=[
["""由于各国货币制度的不同，所以，为了使不同的货币间的清算得以顺利进行，就必须解决各国货币彼此之间的兑换问题，即买卖不同的货币，也就是进行外汇买卖。进行外汇买卖要有一定的场所，这就形成了外汇市场。
狭义的国际外汇市场是指银行间的外汇买卖。外汇银行与客户之间的交易所产生的结果必然有买卖差额，银行本身经过抵冲，其结果可能出现头寸过剩或头寸不足，根据不同的差额，银行就可以在外汇市场上进行抛售或买进，以避免汇率变化所带来的损失。
外汇市场，按有无固定场所分为两种形式：一种是定点交易，凡是参加外汇交易的代表，在一定时间内集中于外汇交易场所进行交易。外汇期货交易就是在交易所内通过公开喊价、拍卖的方式进行。外汇期货市场的首创者是美国芝加哥商品交易所的国际货币市场，它成立于 1972 年 5 月。此外，有固定场所的，还有法国巴黎、德国的法兰克福、比利时的布鲁塞尔等。另一种是非定点交易，即买卖双方通过电话、电传等通讯手段进行交易，如伦敦外汇市场就是这种交易方式。这种交易一般能在 1—2 分钟内结束讨价还价，达成口头协议。目前，除德国、法国等少数国家的外汇市场外，多数国家的外汇市场都脱离了交易所，成为没有固定场所的无形市场。"""],
["""国际商会(International Chamber of Commerce, ICC)是世界重要的民间经贸
组织,成立于1919年，总部设在巴黎。目前在92个国家设有国家委员会，拥有来自122个国家的近10 000家会员公司、协会和个人会员。
国际商会的宗旨是:在经济和法律领域，以有效的行动促进国际贸易和投资的发展。它通过其下设的十几个专业委员会和数十个工作组,制定许多国际商业领域的规则和惯例,如国际贸易术语、国际贸易结算规则等,为全世界广泛采用。国际商会是联合国的重要对话伙伴,并与其他许多重要的国际组织,如世界贸易组织、欧洲联盟、经济合作与发展组织等，保持着密切的关系，对这些组织在制定有关国际商业的政策时有着重要的影响。国际商会为广大商界提供的诸如仲裁、临时进口单证系统、贸易信息网等服务，极大地便利了商界的国际经贸实务操作。
国际商会以贸易为促进和平、繁荣的强大力量,推行一种开放的国际贸易、投资体系和市场经济，且其会员公司和协会皆从事国际商业活动，因此它所制定用以规范国际商业合作的规章，如:《国际贸易术语解释通则》、《托收统一规贝!!》、《跟单信用证统一惯例》等被广泛地应用于国际贸易中，并成为国际贸易不可缺少的一部分。"""],
                ], inputs=[input_text])

if __name__ == "__main__":
    parese = argparse.ArgumentParser()
    parese.add_argument("--port", type=int, default=8081)
    parese.add_argument("--host", type=str, default="0.0.0.0")
    parese.add_argument("--base_llm_url", type=str, default="http://192.168.0.148:8000")
    parese.add_argument("--embedding", type=str, default="/model/bge-large-zh-v1.5")
    
    args = parese.parse_args()
    global base_llm_url,llmchain,vectorstore
    base_llm_url=os.environ.get('LLM_URL_BASE', None)
    if not base_llm_url:
        base_llm_url=args.base_llm_url
    # base_llm=ChatGLMSerLLM(url=base_llm_url)
    # llmchain = LLMChain(llm=base_llm, prompt=CHATGLM_PROMPT,llm_kwargs={"temperature":0.9})
    openai = OpenAI(model_name="chatglm3-6b", openai_api_key="token1",openai_api_base=base_llm_url + "/v1")
    llmchain = LLMChain(llm=openai, prompt=CHATGLM_PROMPT_CT,verbose=True,llm_kwargs={"temperature":1.0})
    
    docs = [Document(page_content=tag) for tag in tags]
    embedding = HuggingFaceEmbeddings(model_name=args.embedding)
    vectorstore = FAISS.from_documents(documents=docs,embedding=embedding)
    
    
    demo.launch(share=False, inbrowser=True,server_name=args.host,server_port=args.port)






