import os
import sys
import time

import requests
sys.path.append("../..") 

from llm.loader import ModelLoader

from common import consts
##👇--------- config -------------
from argparse import Namespace
cfg = Namespace()

cfg.vector = True
cfg.expert = True

cfg.max_source_length = 64 
cfg.max_target_length = 128

cfg.pre_seq_len = 128

#model
cfg.model_name_or_path = consts.MODEL_PATH_ChatGLM2_32K  #远程'THUDM/chatglm-6b' 
cfg.quantization_bit = None #仅仅预测时可以选 4 or 8 
cfg.source_prefix = consts.INSTRUCTION_V1

cfg.checkout_mode = "32k" # lora or ptuning
cfg.ckpt_path = '../../../model/ckpt/chatglm2-6b-32k-qlora-INSv11-rank16-5e-4-30'
cfg.ptuning_path = '../../../model/ckpt/chatglm2-6b-32k-pt-spdsvb-INSv11-128-3e-3-3000/checkpoint-3000'
cfg.output_path='../../../exam/chatglm2-6b-32k-vector'
##👆--------- config end -------------

## --------- load model --------------
if cfg.checkout_mode == "lora":
    # lora 微调 checkpoint 及模型加载
    loader = ModelLoader(cfg.model_name_or_path)
    loader.load_lora(cfg.ckpt_path)
elif cfg.checkout_mode == "ptuning":
    # ptuning v2 微调 checkpoint 及模型加载
    loader = ModelLoader(cfg.model_name_or_path, cfg.pre_seq_len, False)
    loader.load_prefix(cfg.ptuning_path)
else:
    loader = ModelLoader(cfg.model_name_or_path)

model,tokenizer = loader.models()

if cfg.quantization_bit is not None:
    model = loader.quantize(cfg.quantization_bit)

model = model.cuda().eval()

## --------- load model end --------------

## --------- questions --------------
questions = [
    "证书更新时，提示“当前证书和用户绑定非同一证书“，这么处理",
    "提示：该操作需要一个智能卡，但设备中目前没有智能卡，怎么处理",
    "提示：证书库中没有可用的证书，原因是啥",
    "提示：多于一把USBKEY，怎么处理",
    "提示：出现了内部错误，怎么处理",
    "提示：无法在所请求的端口上访问web站点，怎么处理",
    "提示：谷歌浏览器无法反显操作员号，网银助手显示无异常，怎么处理",
    "网银助手提示：没有检测到usbkey中的证书",
    "网银支持的系统及浏览器",
    "网银管理员忘记密码后怎么办",
    "客户想给某一操作员开通手机银行，该如何操作？",
    "网银密码设置规则",
    "网银初始密码如何获取",
    "动账联系人如何维护",
    "登录密码失败最大次数",
    "在柜面给网银客户挂新账号后，在网银做交易下拉框中没有刚挂的账号，是什么原因？",
    "网银做交易时提示“授权模型不匹配”，是什么原因？",
    "客户登录时密码输入框提示需下载安全控件，是什么原因？",
    "客户经办了一笔人民币跨行转账后，发现填写错误，想要修改或撤销该笔交易，在哪个功能下可以执行该操作？",
    "客户经办了一笔人民币跨行转账后，复核人员在哪个功能下可以进行相应的复核操作？",
    "客户想要对某一账号设置日累计限额和笔数，在哪个功能下可以进行相应设置",
]
import pandas as pd
data = pd.DataFrame({"id":[i+1 for i in range(len(questions))],"question":questions})
start = time.time()
# 专家指令
if cfg.expert:
    responses = []
    for q in questions:
        print(f"Q: {q}")    
        prompt = cfg.source_prefix + q
        print("prompt:",prompt)
        response,_=model.chat(tokenizer,prompt,temperature=0)
        responses.append(response)
        # data.loc[data.question==q,"response"] = response
        print(f"A: {response}")
        print("----"*10)
    data["专家指令回答"] = responses
    ins_time = time.time()
    print(f"1 cost time:{ins_time-start}")
    data["评分1"] = [""] * len(questions)


# 知识增强
if cfg.vector:
    responses = []
    for q in questions:
        print(f"Q: {q}")   
        from similarity import get_text_similarity 
        similarity = get_text_similarity(q) 
        if similarity is not None:
            prompt=f"{similarity}\n请结合上述内容回答以下问题：\n{q}" 
        else:
            prompt = cfg.source_prefix + q
        print("prompt:",prompt)
        response,_=model.chat(tokenizer,prompt,temperature=0)
        responses.append(response)
        # data.loc[data.question==q,"response"] = response
        print(f"A: {response}")
        print("----"*10)
    data["知识增强回答"] = responses
    print(f"2 cost time:{time.time()-ins_time}")
    data["评分2"] = [""] * len(questions)

data.to_csv(f"{cfg.output_path}.csv",index=False)


