import json
import os, sys
import re

sys.path.append("../..") 

import time

from common import consts

# -------------- config --------------
from argparse import Namespace

cfg = Namespace()

cfg.source_prefix = consts.INSTRUCTION_V1
####文档问答配置
cfg.embeddings_model='../../../model/moka-ai/m3e-large'
cfg.vectorstore=os.path.join(os.path.expanduser('~'),'.beai/vectorstore')

####知识增强,向量库配置
cfg.enhance_host= os.environ.get('AIGC_VECTOR_HOST', '192.168.22.106')
cfg.enhance_dbname=os.environ.get('AIGC_VECTOR_DBNAME', 'new_vecdoc')
cfg.enhance_username= os.environ.get('AIGC_VECTOR_USER', 'vecdoc')
cfg.enhance_password=os.environ.get('AIGC_VECTOR_PASSWORD', 'vecdoc')
cfg.enhance_port=os.environ.get('AIGC_VECTOR_PORT', '5432')
cfg.enhance_store_path=os.path.join(os.path.expanduser('~'),'.beai/vectorstore_enhance')
cfg.enhance_index_name="know"
cfg.enhance_threshold = 300
cfg.enhance_show_number = 3 #search number = show_number * 3
cfg.enhance_embeddings_model = '../../../model/moka-ai/m3e-large'

# -------------- config end--------------

## ----------- load model --------------
# from llm.loader import ModelLoader
# start = time.time()
# if cfg.checkout_mode == "lora":
#     # lora 微调 checkpoint 及模型加载
#     loader = ModelLoader(cfg.model_name_or_path)
#     loader.load_lora(cfg.ckpt_path)
# elif cfg.checkout_mode == "ptuning":
#     # ptuning v2 微调 checkpoint 及模型加载
#     loader = ModelLoader(cfg.model_name_or_path, cfg.pre_seq_len, False)
#     loader.load_prefix(cfg.ptuning_path)
# else:
#     loader = ModelLoader(cfg.model_name_or_path)

# model,tokenizer = loader.models()

# if cfg.quantization_bit is not None:
#     model = loader.quantize(cfg.quantization_bit)

# model = model.cuda()
# model = model.eval()
# loaded_time = time.time()
# print(f"Load model cost {loaded_time-start:.2f}s")

from llm.chatglm import ChatGLMSerLLM
spdsvb_llm_url=os.environ.get('LLM_URL_SPDSVB', 'http://localhost:8001')
llm=ChatGLMSerLLM(url=spdsvb_llm_url)
## ------------- load model end --------------

# -------------- load base model --------------
# base_loader = ModelLoader(consts.MODEL_PATH_ChatGLM2)
# base_loader.quantize(4)
# base_model,base_tokenizer = base_loader.models()
# base_model.cuda().eval()
base_llm_url=os.environ.get('LLM_URL_BASE', 'http://localhost:8002')
base_llm=ChatGLMSerLLM(url=base_llm_url)
# -------------- load base model end --------------

import flask
from flask import request
app=flask.Flask(__name__)
app.config['JSON_AS_ASCII'] = False
app.config.update(RESTFUL_JSON=dict(ensure_ascii=False))

def re_history(dialog):
    if dialog is None:
        return None
    history=[]
    if len(dialog) == 0:
        return None
    else:
        for item in dialog[::-1]:
            q = re.sub(r"^\[.+?\]", "", item.get("q"))
            history.append((item.get("q"),item.get("a")))
        return history

def parse_text(text):
    """copy from https://github.com/GaiZhenbiao/ChuanhuChatGPT/"""
    lines = text.split("\n")
    lines = [line for line in lines if line != ""]
    count = 0
    for i, line in enumerate(lines):
        if "```" in line:
            count += 1
            items = line.split('`')
            if count % 2 == 1:
                lines[i] = f'<pre><code class="language-{items[-1]}">'
            else:
                lines[i] = f'<br></code></pre>'
        else:
            if i > 0:
                if count % 2 == 1:
                    line = line.replace("`", "\`")
                    line = line.replace("<", "&lt;")
                    line = line.replace(">", "&gt;")
                    line = line.replace(" ", "&nbsp;")
                    line = line.replace("*", "&ast;")
                    line = line.replace("_", "&lowbar;")
                    line = line.replace("-", "&#45;")
                    line = line.replace(".", "&#46;")
                    line = line.replace("!", "&#33;")
                    line = line.replace("(", "&#40;")
                    line = line.replace(")", "&#41;")
                    line = line.replace("$", "&#36;")
                lines[i] = "<br>"+line
    text = "".join(lines)
    return text

from chatbot import Chatbot
from similarity import VectorStore_FAISS
# chatbot=Chatbot(model,tokenizer,base_model,base_tokenizer,source_prefix=cfg.source_prefix,re_history=re_history)

vecstore_faiss = VectorStore_FAISS(
    embedding_model_name=cfg.enhance_embeddings_model,
    store_path=cfg.enhance_store_path,
    index_name=cfg.enhance_index_name,
    info={"port":cfg.enhance_port,"host":cfg.enhance_host,"dbname":cfg.enhance_dbname,"username":cfg.enhance_username,"password":cfg.enhance_password},
    show_number=cfg.enhance_show_number,
    threshold=cfg.enhance_threshold)
chatbot=Chatbot(llm,vecstore_faiss,None,base_llm,None,source_prefix=cfg.source_prefix,re_history=re_history)

@app.route('/aigc/ask', methods=['POST'])
def chat():
    # print("chat")
    data=json.loads(request.get_data(as_text=True))
    # print(data)
    input = data.get('question')
    input = re.sub(r"^\[.+?\]", "", input)

    options=data.get("modelOptions",{})
    isEnhancement=options.get("isEnhancement",False)
    isExpert=options.get("isExpert",False)
    isBase=options.get("isCommon",False)
    temp = float(options.get("sliderTemp", 0.8))
    dialog=data.get("dialog",[])

    # response, history, input = chatbot.chat(input, isExpert, isEnhancement, isBase, data.get("dialog"), temp)
    response, history, input = chatbot.chat_with_llm(input, isExpert, isEnhancement, isBase, dialog, temp)

    # print(history)
    return {
        "code": 200,
        "msg": "success",
        "data": {
            "q": input,	
            "a": parse_text(response),	
        }
    }

@app.route('/user/login', methods=['POST'])
def login():
    data=json.loads(request.get_data(as_text=True))
    print(data)
    return {
        "code": 200,
        "msg": "success",
        "data": "testtoken"
    }

from contract.extraction import ElementsExtractor
from contract.documentqa import DocumentQA
from contract.callbacks import PrintRetrievalHandler
# from llm.wrapper import WrapperLLM
# llm=WrapperLLM(model=model,tokenizer=tokenizer)
extractor=ElementsExtractor(llm=llm)
_documentqa=DocumentQA(llm=base_llm)

@app.route('/aigc/extract', methods=['POST'])
def extract():
    # 从请求中获取数据
    data = json.loads(request.form['params'])
    print(data)

    # 读取上传的文件
    file = request.files['file']
    if file is None:
        return {"code": 0, 'msg': 'Error: could not load file'}

    suffix = file.filename.split(".")[-1]
    from tempfile import NamedTemporaryFile
    temp_file = NamedTemporaryFile(suffix=f".{suffix}", delete=False)
    temp_file.write(file.read())
    temp_file.close()
    filename = temp_file.name

    # 获取模型选项中的参数
    sliderAnswer = data.get("modelOptions").get("sliderAnswer")
    sliderAnswer = int(float(sliderAnswer))

    elements = data.get("tags")

    from  extract import contract
    result = contract(extractor,filename,elements,8000)

    os.unlink(filename)

    # 返回提取结果
    return {
        "code": 200,
        "msg": "success",
        "data": [[k, v] for k, v in result.items()]
    }

@app.route('/aigc/docqa', methods=['POST'])
def documentqa():
    # 从请求中获取数据
    data = json.loads(request.form['params'])
    print(data)
    
    # 读取上传的文件
    filenames=[]
    if request.files:           #基于chatid的回话文件非必输
        for _,file in request.files.items():
            suffix = os.path.splitext(file.filename)[1]
            prefix = os.path.splitext(os.path.basename(file.filename))[0]
            from tempfile import NamedTemporaryFile
            temp_file = NamedTemporaryFile(prefix = prefix,suffix=f"{suffix}", delete=False)
            temp_file.write(file.read())
            temp_file.close()
            filenames.append(temp_file.name)
    #根据chatid载入向量库
    chatid=data.get("chatid")
    answer=""


    query=data.get("query")
    detail=data.get("detail",True)
    summary=data.get("summary",False)
    callback_doc = None
        #暂时只开放QA
    _faiss = VectorStore_FAISS(embedding_model_name=cfg.embeddings_model,show_number=cfg.enhance_show_number,index_name=chatid,store_path=cfg.vectorstore,is_pgsql=False,threshold=cfg.enhance_threshold)
    if len(filenames) > 0:
        _faiss._add_documents_from_dir(filepaths=filenames,load_kwargs={"sentence_size":1024})
        _faiss._save_local()
    if detail:
        callback_doc = PrintRetrievalHandler()
    if (query is None or len(query.strip()) == 0) and summary:       
        answer= _documentqa.summarize_document(filepaths=filenames,load_kwargs={"sentence_size":2048},chain_type="map_reduce", chain_type_kwargs={"token_max":3072,"verbose":True})
    else:

        answer=_documentqa.qa_from_document(query=data.get("query"),
                                            retriever=_faiss.as_retriever(),
                                            chain_type=data.get("chain_type") or "map_reduce",
                                            chain_type_kwargs={"verbose":True},
                                            callbacks=[callback_doc] if callback_doc else None)
    # answer = _documentqa.route_chain(query=data.get("query"),filepaths=filenames,load_kwargs={"sentence_size":1000},embeddings_model=cfg.embeddings_model,chain_type=data.get("chain_type") or "refine",chain_type_kwargs={"verbose":True})
    # if data.get("type") == "qa":
    #     answer=_documentqa.qa_from_document(query=data.get("query"),retriever=GetRetriever(cfg.embeddings_model,filepaths=filenames,vectorstorepath=cfg.vectorstore,index_name=chatid,load_kwargs={"sentence_size":1000},k=10),chain_type=data.get("chain_type") or "refine",chain_type_kwargs={"verbose":True})
    # elif data.get("type") == "sum":
    #     answer=_documentqa.summarize_document(filepaths=filenames,chain_type=data.get("chain_type") or "refine",load_kwargs={"sentence_size":1000},chain_type_kwargs={"verbose":True})
    # elif data.get("type") == "extract":
    #     answer=_documentqa.extract_from_document(query=data.get("query"),filepaths=filename,chain_type=data.get("chain_type") or "refine",chain_type_kwargs={"verbose":True},load_kwargs={"sentence_size":1000})
    # else:
    #     return {
    #         "code": 0,
    #         "msg": f"Unknown request type:%s" % data.get("type")
    #     }
    for filename in filenames:
        os.unlink(filename)
    print(answer)
    return {
        "code": 200,
        "msg": "success",
        "data": {
            "q": data.get("query"),	
            "a": parse_text(answer),
            "similarity":callback_doc.getsimilarity() if callback_doc else []
        }
    }

from contract.documentqa import  del_vectorstore_path

@app.route('/aigc/docqa',methods=['DELETE'])
def delete():
    data = json.loads(request.get_data(as_text=True))
    chatid=data.get("chatid")
    del_vectorstore_path(chatid, cfg.vectorstore)

    return {
        "code": 200,
        "msg": "success",
        "data": "delete success"
    }

def main():
    app.run(host="0.0.0.0",port=5000,debug=False)

if __name__ == "__main__":
    main()