import os, sys
import re

sys.path.append("../..") 

from contract.documentqa import DocumentQA,GetEmbding,GetRetriever
from llm.chatglm import ChatGLMSerLLM
from llm.ernie_with_sdk import ChatERNIESerLLM
from loader.load import load,loads_path,loads,append
from langchain.embeddings.huggingface import (
    HuggingFaceEmbeddings,
)
from langchain.chains import RetrievalQA
from langchain.vectorstores import FAISS
from contract.prompts import (
    QA_PROMPT,
    REFINE_QA_PROMPT,
    SUMMARISE_PROMPT,
    REFINE_SUMMARISE_PROMPT,
    EXTRACTION_PROMPT,
    REFINE_EXTRACTION_PROMPT,
    ROUTER_PROMPT,
    # CHAT_QUESTION_PROMPT,
    # CHAT_COMBINE_PROMPT
)
from langchain.callbacks.base import (
    BaseCallbackHandler,
    BaseCallbackManager
)

class PrintRetrievalHandler(BaseCallbackHandler):

    def on_retriever_start(self, query: str, **kwargs):
        print(f"**Question:** {query}")

    def on_retriever_end(self, documents, **kwargs):
        for idx, doc in enumerate(documents):
            source = doc.metadata["source"]
            print(f"**Results from {source}**")
            print(doc.page_content)


base_llm=ChatGLMSerLLM(url="http://192.168.22.106:8002")
# base_llm=ChatGLMSerLLM(url="http://192.168.0.148:8000")
documentqa = DocumentQA(llm=base_llm)

def test_qa():
    filepath = "/dataset/浦发硅谷电子渠道、企业网银、APIbanking、电子回单系统运维专业服务合同(4).docx"
    # embedding_path = '/model/text2vec/text2vec-base-chinese'
    embedding_path = '/model/moka-ai/m3e-large'
    sentence_size = 1024
    # retriever=GetRetriever(embdingpath=embedding_path,filepaths=[filepath],
    #                        load_kwargs={"sentence_size":512},
    #                        save_local=False,
    #                        search_kwargs={"k":5})
                        #    search_type="similarity_score_threshold",search_kwargs={"score_threshold":200.0})
    # result = documentqa.summarize_document(filepaths=[filepath],load_kwargs={"sentence_size":sentence_size},chain_type="refine", chain_type_kwargs={"verbose":True},temperature=0.8)
    start=time.time()
    documents = loads(filepaths=[filepath],sentence_size=sentence_size*4/3)
    # print("document_len",[len(doc.page_content) for doc in documents])
    # print("documents size",len(documents))
    # documents = append(documents=documents,sentence_size=sentence_size)
    # print("document_len",[len(doc.page_content) for doc in documents])
    # print("documents size",len(documents))
    # documents = retriever._get_relevant_documents(query="受托人",run_manager=None)
    # print([doc.page_content for doc in documents])
    result = documentqa.summarize_document(documents=documents,chain_type="map_reduce", chain_type_kwargs={"token_max":sentence_size,"verbose":True})
    # result = documentqa.qa_from_document(query="合同中的“受托人”名称",retriever=retriever,chain_type="map_reduce", chain_type_kwargs={"token_max":3000,"verbose":True},callbacks=[PrintRetrievalHandler()])
    # result = documentqa.qa_from_document(query="合同服务期限",retriever=retriever,chain_type="refine", chain_type_kwargs={"verbose":True},callbacks=[PrintRetrievalHandler()])

    # qa = RetrievalQA.from_chain_type(llm=base_llm, chain_type="map_reduce", 
    #                                  chain_type_kwargs={"question_prompt":CHAT_QUESTION_PROMPT,"combine_prompt":CHAT_COMBINE_PROMPT,"token_max":3000,"verbose":True},retriever=retriever)

    # result = qa.run("合同中的委托人公司名称和受托人公司名称分别是")
    cost_time=time.time()-start
    # print("document_len",[len(doc.page_content) for doc in documents])
    # print("documents size",len(documents))
    print("cost_time",cost_time)
    print(result)
    
from vector.pgsql.db import PostgresDB
from similarity import VectorStore_FAISS
from langchain.schema import Document
    
def test_faiss():
    vecstore_faiss = VectorStore_FAISS(
    embedding_model_name='../../../model/moka-ai/m3e-large',
    store_path=os.path.join(os.path.expanduser('~'),'.beai/vectorstore_enhance'),
    index_name="know",
    info={"port":"5432","host":"192.168.22.106","dbname":"new_vecdoc","username":"vecdoc","password":"vecdoc"},
    show_number=3,
    reset=True)
    
    psqldb = PostgresDB("192.168.22.106", "vecdoc", "vecdoc", "vecdoc")
    psqldb.connect()
    #将当前向量库中的数据全部导出到新的向量库中
    db_index = 0
    page_size = 2000
    print(vecstore_faiss._faiss.index.ntotal)
    while True:
        # query = f"SELECT text,paragraph_id FROM vec_txt order by vector_id limit %s offset %s" % (page_size,db_index)
        query = f"select vec_txt.text,vec_txt.paragraph_id,txt_doc.text,count(*) as count from vec_txt left join txt_doc on vec_txt.paragraph_id=txt_doc.paragraph_id where vec_txt.text not like E'%%\n%%' and vec_txt.text != '' and vec_txt.text != ' ' and vec_txt.paragraph_id in(select DISTINCT on (text) paragraph_id from txt_doc order by text,paragraph_id) group by vec_txt.text,txt_doc.text,vec_txt.paragraph_id limit %s offset %s" % (page_size,db_index)
        psqldb.execute(query)
        questions = psqldb.fetchall()
        if len(questions) <= 0:
            break
        db_index+=page_size
        list_of_documents = []
        for question in questions:
            list_of_documents.append(Document(page_content=question[0], metadata=dict(paragraph=question[2],page=question[1])))
        vecstore_faiss._add_documents(list_of_documents,need_split=False)
        print(vecstore_faiss._faiss.index.ntotal)
        
    vecstore_faiss._save_local()
from langchain.document_loaders import UnstructuredFileLoader, TextLoader, CSVLoader,UnstructuredPDFLoader,UnstructuredWordDocumentLoader
def test_faiss_from_dir():
    vecstore_faiss = VectorStore_FAISS(
    embedding_model_name='../../../model/moka-ai/m3e-large',
    store_path=os.path.join(os.path.expanduser('~'),'.beai/vectorstore_enhance'),
    index_name="know",
    info={"port":5434,"host":"192.168.22.106","dbname":"vecdoc","username":"vecdoc","password":"vecdoc"},
    show_number=3,
    reset=True)
    docs = loads_path("../../../data/docs",mode="elements",sentence_size=1024)
    print(len(docs))
    docs = vecstore_faiss._tuple_deduplication(docs)
    print(len(docs))
    print(vecstore_faiss._faiss.index.ntotal)
    for i in range(0, len(docs), 300):
        vecstore_faiss._add_documents(docs[i:i+300 if i+300<len(docs) else len(docs)],need_split=True)
        print(vecstore_faiss._faiss.index.ntotal)
    vecstore_faiss._save_local()
    
def test_faiss_load():
    vecstore_faiss = VectorStore_FAISS(
    embedding_model_name='../../../model/moka-ai/m3e-large',
    store_path=os.path.join(os.path.expanduser('~'),'.beai/vectorstore_enhance'),
    index_name="know",
    info={"port":5432,"host":"192.168.22.106","dbname":"new_vecdoc","username":"vecdoc","password":"vecdoc"},
    show_number=3,
    reset=False)
    print(vecstore_faiss._join_document(vecstore_faiss.get_text_similarity_with_score("通知存款支取的业务规则")))
if __name__ == "__main__":
    test_faiss_from_dir()
    

