import logging
import os, sys
sys.path.append("../..") 
logging.basicConfig(filename='web.log', level=logging.INFO)
logger = logging.getLogger(__name__)
from common import consts

import gradio as gr
import mdtex2html

import torch
import transformers
from transformers import (
    AutoConfig,
    AutoModel,
    AutoTokenizer,
    AutoTokenizer,
    DataCollatorForSeq2Seq,
    HfArgumentParser,
    Seq2SeqTrainingArguments,
    set_seed,
)

from argparse import Namespace

from llm.loader import ModelLoader
cfg = Namespace()

#model
cfg.checkout_mode = "lora" # lora or ptuning
cfg.model_name_or_path = consts.MODEL_PATH_ChatGLM2_32K # 32K 利于知识增强模式
# cfg.model_name_or_path = '/home/zfh/models/tunning/chatglm2-6b-lora-spdsvb'
cfg.ptuning_path = '/home/zfh/aird/model/ckpt/chatglm-6b-pt-spdsvb-INSv8-128-5e-3-3000/checkpoint-3000'
cfg.ckpt_path = '/home/zfh/aird/model/ckpt/chatglm-6b-lora-spdsvb-INSv10-1e-03-20'
cfg.pre_seq_len = 0 
# cfg.pre_seq_len = 128
cfg.prefix_projection = False
cfg.quantization_bit = None

cfg.source_prefix = consts.INSTRUCTION_V1

## --------- load model --------------
if cfg.checkout_mode == "lora":
    # lora 微调 checkpoint 及模型加载
    loader = ModelLoader(cfg.model_name_or_path)
    loader.load_lora(cfg.ckpt_path)
elif cfg.checkout_mode == "ptuning":
    # ptuning v2 微调 checkpoint 及模型加载
    loader = ModelLoader(cfg.model_name_or_path, cfg.pre_seq_len, False)
    loader.load_prefix(cfg.ptuning_path)

model,tokenizer = loader.models()

if cfg.quantization_bit is not None:
    model = loader.quantize(cfg.quantization_bit)

model = model.cuda().eval()

## --------- load model end --------------

"""Override Chatbot.postprocess"""
def postprocess(self, y):
    if y is None:
        return []
    for i, (message, response) in enumerate(y):
        y[i] = (
            None if message is None else mdtex2html.convert((message)),
            None if response is None else mdtex2html.convert(response),
        )
    return y


gr.Chatbot.postprocess = postprocess


def parse_text(text):
    """copy from https://github.com/GaiZhenbiao/ChuanhuChatGPT/"""
    lines = text.split("\n")
    lines = [line for line in lines if line != ""]
    count = 0
    for i, line in enumerate(lines):
        if "```" in line:
            count += 1
            items = line.split('`')
            if count % 2 == 1:
                lines[i] = f'<pre><code class="language-{items[-1]}">'
            else:
                lines[i] = f'<br></code></pre>'
        else:
            if i > 0:
                if count % 2 == 1:
                    line = line.replace("`", "\`")
                    line = line.replace("<", "&lt;")
                    line = line.replace(">", "&gt;")
                    line = line.replace(" ", "&nbsp;")
                    line = line.replace("*", "&ast;")
                    line = line.replace("_", "&lowbar;")
                    line = line.replace("-", "&#45;")
                    line = line.replace(".", "&#46;")
                    line = line.replace("!", "&#33;")
                    line = line.replace("(", "&#40;")
                    line = line.replace(")", "&#41;")
                    line = line.replace("$", "&#36;")
                lines[i] = "<br>"+line
    text = "".join(lines)
    return text

# def predict(input, chatbot, max_length, top_p, temperature, enhance, instruct,history):
def predict(input, chatbot, max_length, enhance, instruct, history):
    # chatbot.append((parse_text(input), ""))
    if enhance:
        print("知识增强")
        history = []
        from similarity import get_text_similarity
        similarity = get_text_similarity(input)
        print("similarity:",similarity)
        if similarity is not None :
            prompt=f"{similarity}\n请结合上述内容回答以下问题：\n{input}" 
        elif cfg.source_prefix is not None and instruct:
            prompt = cfg.source_prefix + input
        else:
            prompt = f"对不起，我没有找到相关内容\n请结合上述内容回答以下问题：\n{input}" 
        input = f"[知识增强]{input}"
    elif instruct:
        print("专家指令")
        if cfg.source_prefix is not None and instruct:
            history = []
            prompt = cfg.source_prefix + input
        else:
            prompt = input
        input = f"[专家指令]{input}"
    else:
        print("普通问答")
        prompt = input
        input = f"[普通问答]{input}"
    chatbot.append((input, ""))
    logger.info(f"prompt: {prompt}")
    # for response, history in model.stream_chat(tokenizer, prompt, history, max_length=max_length, top_p=top_p, # type: ignore
    #                                            temperature=temperature):
    for response, history in model.stream_chat(tokenizer, prompt, history, max_length=max_length,  # type: ignore
                                               ):
        chatbot[-1] = (parse_text(input), parse_text(response))       

        yield chatbot, history

def reset_user_input():
    return gr.update(value='')


def reset_state():
    return [], []


with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">ChatGLM - SPDSVB</h1>""")

    chatbot = gr.Chatbot()
    with gr.Row():
        with gr.Column(scale=4):
            with gr.Column(scale=12):
                user_input = gr.Textbox(show_label=False, placeholder="Input...", lines=10)
            with gr.Column(min_width=32, scale=1):
                submitBtn = gr.Button("Submit", variant="primary")
        with gr.Column(scale=1):
            emptyBtn = gr.Button("Clear History")
            max_length = gr.Slider(0, 32768, value=2048, step=100.0, label="Maximum length", interactive=True)
            if cfg.model_name_or_path == consts.MODEL_PATH_ChatGLM2_32K:
                max_length.value = 32768
            # top_p = gr.Slider(0, 1, value=0.7, step=0.01, label="Top P", interactive=True)
            # temperature = gr.Slider(0, 1, value=0.95, step=0.01, label="Temperature", interactive=True)
            with gr.Row():
                enhance = gr.Checkbox(label="知识增强", interactive=True)
                instruct = gr.Checkbox(label="专家指令",value=True, interactive=True)

    history = gr.State([])

    # submitBtn.click(predict, [user_input, chatbot, max_length, top_p, temperature, enhance, instruct, history], [chatbot, history],
    submitBtn.click(predict, [user_input, chatbot, max_length, enhance, instruct, history], [chatbot, history],
                    show_progress="minimal")
    submitBtn.click(reset_user_input, [], [user_input])

    emptyBtn.click(reset_state, outputs=[chatbot, history], show_progress='minimal')



def main():
    demo.queue().launch(share=True,server_port=9999)

if __name__ == "__main__":
    main()