import os, sys
sys.path.append("../..") 

import logging
logging.basicConfig(filename='web.log', level=logging.INFO)
logger = logging.getLogger(__name__)

from dotenv import load_dotenv
load_dotenv()

from common import consts

import gradio as gr
import mdtex2html

from llm.spark import SparkLLM
from langchain.chat_models import ChatOpenAI
from qa.generator import QAGenerator

llm=SparkLLM()
generator=QAGenerator(llm=llm)



def parse_text(text):
    """copy from https://github.com/GaiZhenbiao/ChuanhuChatGPT/"""
    lines = text.split("\n")
    lines = [line for line in lines if line != ""]
    count = 0
    for i, line in enumerate(lines):
        if "```" in line:
            count += 1
            items = line.split('`')
            if count % 2 == 1:
                lines[i] = f'<pre><code class="language-{items[-1]}">'
            else:
                lines[i] = f'<br></code></pre>'
        else:
            if i > 0:
                if count % 2 == 1:
                    line = line.replace("`", "\`")
                    line = line.replace("<", "&lt;")
                    line = line.replace(">", "&gt;")
                    line = line.replace(" ", "&nbsp;")
                    line = line.replace("*", "&ast;")
                    line = line.replace("_", "&lowbar;")
                    line = line.replace("-", "&#45;")
                    line = line.replace(".", "&#46;")
                    line = line.replace("!", "&#33;")
                    line = line.replace("(", "&#40;")
                    line = line.replace(")", "&#41;")
                    line = line.replace("$", "&#36;")
                lines[i] = "<br>"+line
    text = "".join(lines)
    return text

# def predict(input, chatbot, max_length, top_p, temperature, enhance, instruct,history):
def predict(input, chatbot, max_length, history):
    # chatbot.append((parse_text(input), ""))
    result=[]
    questions = generator.generate_questions(input,question_number=3)
    # print("questions:",questions)
    for q in questions:
        answer=generator.generate_answer(input,q)
        result.append((q,answer))
        # print("Q:",q,"A:",answer)
    response = "\n\n".join([f"{q}\n{a}" for q,a in result])
    # print("response:",response)
    chatbot.append((parse_text(response),""))
    return chatbot, history

def reset_user_input():
    return gr.update(value='')


def reset_state():
    return [], []


def change_model(model_name):
    global llm, generator
    if model_name == "spark":
        llm=SparkLLM()
    elif model_name == "openai":
        llm=ChatOpenAI(model_name="gpt-3.5-turbo")
    generator=QAGenerator(llm=llm)
    return model_name

with gr.Blocks() as demo:
    gr.HTML("""<h1 align="center">训练集提取</h1>""")

    chatbot = gr.Chatbot()
    with gr.Row():
        with gr.Column(scale=4):
            with gr.Column(scale=12):
                user_input = gr.Textbox(show_label=False, placeholder="Input...", lines=10)
            with gr.Column(min_width=32, scale=1):
                submitBtn = gr.Button("Submit", variant="primary")
        with gr.Column(scale=1):
            model_name = gr.Radio(choices=["spark", "openai"], label="Model", value="spark", inline=True,interactive=True)
            max_length = gr.Slider(0, 32768, value=2048, step=100.0, label="Maximum length", interactive=True)
            emptyBtn = gr.Button("Clear History")
            # top_p = gr.Slider(0, 1, value=0.7, step=0.01, label="Top P", interactive=True)
            # temperature = gr.Slider(0, 1, value=0.95, step=0.01, label="Temperature", interactive=True)
        model_name.change(change_model,[model_name],[model_name])

    history = gr.State([])

    # submitBtn.click(predict, [user_input, chatbot, max_length, top_p, temperature, enhance, instruct, history], [chatbot, history],
    submitBtn.click(predict, [user_input, chatbot, max_length, history], [chatbot, history],
                    show_progress="minimal")
    submitBtn.click(reset_user_input, [], [user_input])

    emptyBtn.click(reset_state, outputs=[chatbot, history], show_progress='minimal')



def main():
    demo.queue().launch(share=True,server_port=9999)

if __name__ == "__main__":
    main()