import json
import matplotlib.pyplot as plt

# 加载 json 文件
# 读取其中的 log_history 列表数据
# 基于 log_history 列表数据，绘制 loss 曲线图
def load_json(file_path):
    with open(file_path, 'r') as f:
        data = json.load(f)
        return data['log_history']
        
def plot_loss(log_history,label):
    x,y=[],[]
    ex,ey=[],[]
    # for i in range(len(log_history)):
    for item in log_history:
        if 'loss' in item:
            x.append(item["step"])
            y.append(item["loss"])
        elif 'eval_loss' in item:
            ex.append(item["step"])
            ey.append(item["eval_loss"])
        else:
            # print('Error: loss key not found in item:', item)
            continue

    plt.plot(x, y,'-o', label=label+"train_loss")
    if len(ey)>0:
        plt.plot(ex, ey,'-o', label=label+"eval_loss")

    # plt.scatter(x, y, label='train_loss')
    plt.legend()
# plt.show(block=True)

def create_plot(dataset,name):
    plt.figure(figsize=(10, 4))
    for item in dataset:
        for key,value in item.items():
            log_history = load_json(value)
            plot_loss(log_history,key)
    plt.savefig(f'../../images/loss/{name}.png')

def create_plot_one(state_file):
    plt.figure(figsize=(10, 4))
    log_history = load_json(state_file)
    dir = '/'.join(state_file.split('/')[:-1])
    plot_loss(log_history,"")
    plt.savefig(f'{dir}/loss.png')

dataset_chatglm = [
    # {"adgen-chatglm-6b-pt-comb-128-2e-2":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-comb-128-2e-2/trainer_state.json'},
    # {"adgen-chatglm-6b-pt-comb-128-2e-2-3000":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-comb-128-2e-2-3000/trainer_state.json'},
    # {"adgen-chatglm-6b-pt-comb-128-5e-3-1000":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-comb-128-5e-3-1000/trainer_state.json'},
    # {"adgen-chatglm-6b-pt-comb-128-1e-3-3000":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-comb-128-1e-3-3000/trainer_state.json'},
    {"adgen-chatglm-6b-pt-spdsvb-128-1e-2-3000":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-spdsvb-128-1e-3-3000/trainer_state.json'},
    {"adgen-chatglm-6b-pt-spdsvb-128-1e-3-3000-v2":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-spdsvb-128-1e-3-3000-v2/trainer_state.json'},
    {"adgen-chatglm-6b-pt-spdsvb-128-1e-3-3000-v3":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-spdsvb-128-1e-3-3000-v3/trainer_state.json'},
    {"adgen-chatglm-6b-pt-spdsvb-128-5e-3-3000-base":'/home/zfh/ChatGLM/ChatGLM-6B/ptuning/output/adgen-chatglm-6b-pt-128-5e-3-3000-spdsvb-base/trainer_state.json'},
    # {"adgen-chatglm2-6b-pt-128-5e-4-spdsvb-v3":"/home/zfh/ChatGLM/ChatGLM2-6B/ptuning/output/adgen-chatglm2-6b-pt-128-5e-4-spdsvb-v3/trainer_state.json"},
    # {"adgen-chatglm2-6b-pt-128-5e-4-spdsvb-base":"/home/zfh/ChatGLM/ChatGLM2-6B/ptuning/output/adgen-chatglm2-6b-pt-128-5e-4-spdsvb-base/trainer_state.json"},
    {"chatglm-6b-pt-spdsvb-INSv8-128-5e-3-3000":"/home/zfh/aird/model/ckpt/chatglm-6b-pt-spdsvb-INSv8-128-5e-3-3000/trainer_state.json"},
    {"chatglm-6b-pt-spdsvb-INSv9-128-5e-3-3000":"/home/zfh/aird/model/ckpt/chatglm-6b-pt-spdsvb-INSv9-128-5e-3-3000/trainer_state.json"},
    {"chatglm-6b-pt-spdsvb-INSv10-128-5e-3-3000":"/home/zfh/aird/model/ckpt/chatglm-6b-pt-spdsvb-INSv10-128-5e-3-3000/trainer_state.json"},
]

dataset_v11 = [
    {"chatglm2-6b-pt-spdsvb-INSv11-128-1e-3-3000":"/home/zfh/aird/model/ckpt/chatglm2-6b-pt-spdsvb-INSv11-128-1e-3-3000/trainer_state.json"},
    {"chatglm2-6b-pt-spdsvb-INSv11-128-5e-3-3010":"/home/zfh/aird/model/ckpt/chatglm2-6b-pt-spdsvb-INSv11-128-5e-3-3010/trainer_state.json"},
    {"chatglm2-6b-qlora-INSv11-rank16-1e-3-30":"/home/zfh/aird/model/ckpt/chatglm2-6b-qlora-INSv11-rank16-1e-3-30/checkpoint-2000/trainer_state.json"},
    {"chatglm2-6b-pt-spdsvb-INSv11-128-3e-3-1000+s1000":"/home/zfh/aird/model/ckpt/chatglm2-6b-pt-spdsvb-INSv11-128-3e-3-1000+s1000/trainer_state.json"},
    {"chatglm2-6b-32k-qlora-INSv11-rank16-5e-4-30":"/home/zfh/aird/model/ckpt/chatglm2-6b-32k-qlora-INSv11-rank16-5e-4-30/checkpoint-2800/trainer_state.json"},
    {"chatglm2-6b-32k-pt-spdsvb-INSv11-128-3e-3-3000":"/home/zfh/aird/model/ckpt/chatglm2-6b-32k-pt-spdsvb-INSv11-128-3e-3-3000/checkpoint-3000/trainer_state.json"}
]


dataset_lora = [
    # {"chatglm-6b-lora-spdsvb-base":'/home/zfh/aird/model/ckpt/chatglm-6b-lora-spdsvb-base/train_history.csv'},
    {"chatglm2-6b-qlora-INSv11_rank16-1e-3-30":"/home/zfh/aird/model/ckpt/chatglm2-6b-qlora-INSv11_rank16-1e-3-30/checkpoint-2000/trainer_state.json"},
]

create_plot(dataset_chatglm,'spdsvb')
# create_plot(dataset_chatglm2,'chatglm2')

for item in dataset_chatglm:
    for key,value in item.items():
        create_plot_one(value)

create_plot(dataset_v11,'spdsvb_v11')
for item in dataset_v11:
    for key,value in item.items():
        create_plot_one(value)
        