import sys
sys.path.append("../..") 

from common import consts
## --------- config -------------
# from argparse import Namespace
# cfg = Namespace()

# #dataset
# cfg.train_file = '../../../data/train_spdsvb_v10.csv' 
# cfg.val_file = '../../../data/val_spdsvb_v4.csv' 
# cfg.prompt_column = 'prompt'
# cfg.response_column = 'response'
# cfg.history_column = None
# cfg.source_prefix = consts.INSTRUCTION_V1 #添加到每个prompt开头的前缀引导语

# cfg.max_source_length = 64 
# cfg.max_target_length = 128

# #model
# cfg.model_name_or_path = consts.MODEL_PATH_ChatGLM2_32K  #远程'THUDM/chatglm-6b' 
# cfg.quantization_bit = None #仅仅预测时可以选 4 or 8 

# #train
# cfg.epochs = 20
# cfg.lr = 1e-3
# cfg.batch_size = 4
# cfg.gradient_accumulation_steps = 16 #梯度累积

# cfg.ckpt_path =  '../../../model/ckpt/chatglm-6b-lora-spdsvb-INSv10-{:.0e}-{}'.format(cfg.lr, cfg.epochs)

import argparse

parser = argparse.ArgumentParser(description='Lora Chatbot Training')

# dataset
parser.add_argument('--train_file', type=str, default='../../../data/train_spdsvb_v10.csv', help='path to training dataset')
parser.add_argument('--val_file', type=str, default='../../../data/val_spdsvb_v4.csv', help='path to validation dataset')
parser.add_argument('--prompt_column', type=str, default='prompt', help='name of the prompt column')
parser.add_argument('--response_column', type=str, default='response', help='name of the response column')
parser.add_argument('--history_column', type=str, default=None, help='name of the history column')
parser.add_argument('--source_prefix', type=str, default=consts.INSTRUCTION_V1, help='prefix added to each prompt')

parser.add_argument('--max_source_length', type=int, default=64, help='maximum length of the input sequence')
parser.add_argument('--max_target_length', type=int, default=128, help='maximum length of the output sequence')

# model
parser.add_argument('--model_name_or_path', type=str, default=consts.MODEL_PATH_ChatGLM2_32K, help='path to the pre-trained model')
parser.add_argument('--quantization_bit', type=int, default=None, help='number of bits for quantization (4 or 8)')

# train
parser.add_argument('--epochs', type=int, default=20, help='number of training epochs')
parser.add_argument('--lr', type=float, default=1e-3, help='learning rate')
parser.add_argument('--batch_size', type=int, default=4, help='batch size')
parser.add_argument('--gradient_accumulation_steps', type=int, default=16, help='number of gradient accumulation steps')
parser.add_argument('--ckpt_path', type=str, default="", help='path to save the trained model')

cfg = parser.parse_args()
## --------- end config -------------


## --------- load model -------------
import transformers
from transformers import  AutoModel,AutoTokenizer,AutoConfig,DataCollatorForSeq2Seq

config = AutoConfig.from_pretrained(cfg.model_name_or_path, trust_remote_code=True)

tokenizer = AutoTokenizer.from_pretrained(
    cfg.model_name_or_path, trust_remote_code=True)

model = AutoModel.from_pretrained(cfg.model_name_or_path,config=config,
                                  trust_remote_code=True).half() 

#先量化瘦身
if cfg.quantization_bit is not None:
    print(f"Quantized to {cfg.quantization_bit} bit")
    model = model.quantize(cfg.quantization_bit)
    
#再移动到GPU上
model = model.cuda()
## --------- end load model -------------


## --------- load data -------------
def preprocess(examples):
    max_seq_length = cfg.max_source_length + cfg.max_target_length
    model_inputs = {
        "input_ids": [],
        "labels": [],
    }
    for i in range(len(examples[cfg.prompt_column])):
        if examples[cfg.prompt_column][i] and examples[cfg.response_column][i]:
            query, answer = examples[cfg.prompt_column][i], examples[cfg.response_column][i]

            history = examples[cfg.history_column][i] if cfg.history_column is not None else None
            # prompt = tokenizer.build_prompt(query, history)
            prompt = query

            prompt = cfg.source_prefix + prompt
            a_ids = tokenizer.encode(text=prompt, add_special_tokens=True, truncation=True,
                                     max_length=cfg.max_source_length)
            b_ids = tokenizer.encode(text=answer, add_special_tokens=False, truncation=True,
                                     max_length=cfg.max_target_length)

            context_length = len(a_ids)
            input_ids = a_ids + b_ids + [tokenizer.eos_token_id]
            labels = [tokenizer.pad_token_id] * context_length + b_ids + [tokenizer.eos_token_id]

            pad_len = max_seq_length - len(input_ids)
            input_ids = input_ids + [tokenizer.pad_token_id] * pad_len
            labels = labels + [tokenizer.pad_token_id] * pad_len
            labels = [(l if l != tokenizer.pad_token_id else -100) for l in labels]
            model_inputs["input_ids"].append(input_ids)
            model_inputs["labels"].append(labels)
    return model_inputs

from datasets import load_dataset

data_files = {"train": cfg.train_file, "val": cfg.val_file}
data_sets = load_dataset(cfg.train_file.split(".")[-1], data_files=data_files)

ds_train = ds_val = None
if cfg.train_file is not None:
    ds_train = data_sets["train"].map(preprocess, batched=True,remove_columns=data_sets["train"].column_names)
    print(data_sets["train"])
if cfg.val_file is not None:
    ds_val = data_sets["val"].map(preprocess, batched=True,remove_columns=data_sets["val"].column_names)
    print(data_sets["val"])


data_collator = DataCollatorForSeq2Seq(
    tokenizer,
    model=None,
    label_pad_token_id=-100,
    pad_to_multiple_of=None,
    padding=False
)

from torch.utils.data import DataLoader
dl_train = DataLoader(ds_train, batch_size = cfg.batch_size,
                      num_workers = 2, shuffle = True, collate_fn = data_collator 
                     )
dl_val = DataLoader(ds_val, batch_size = cfg.batch_size,
                      num_workers = 2, shuffle = False, collate_fn = data_collator 
                     )
## --------- end load data -------------



## --------- train -------------
from peft import get_peft_model, AdaLoraConfig, TaskType

#训练时节约GPU占用
model.config.use_cache=False
model.supports_gradient_checkpointing = True  #
model.gradient_checkpointing_enable()
model.enable_input_require_grads()

peft_config = AdaLoraConfig(
    task_type=TaskType.CAUSAL_LM, inference_mode=False,
    r=8,
    lora_alpha=32, lora_dropout=0.1,
    target_modules=["query", "value"]
)

peft_model = get_peft_model(model, peft_config)

peft_model.is_parallelizable = True
peft_model.model_parallel = True
peft_model.print_trainable_parameters()


from keras_model import KerasModel
import torch

optimizer = torch.optim.AdamW(peft_model.parameters(),lr=cfg.lr) 
keras_model = KerasModel(peft_model, loss_fn = None, optimizer=optimizer) 

df = keras_model.fit(train_data = dl_train,
                val_data = dl_val,
                epochs=cfg.epochs,
                patience=20,
                monitor='val_loss',
                mode='min',
                ckpt_path = cfg.ckpt_path,
                mixed_precision='fp16',
                gradient_accumulation_steps = cfg.gradient_accumulation_steps
               )
df.to_json(f"{cfg.ckpt_path}/train_history.json")


