
from langchain.vectorstores import FAISS
from langchain.embeddings.huggingface import (
    HuggingFaceEmbeddings,
)
import faiss
from pgsql.db import PostgresDB

from langchain.schema import Document
from pgsqldocstore import PgSqlDocstore        

docstore1 = PgSqlDocstore(["192.168.0.148","new_vecdoc","vecdoc","vecdoc"],reset=True)
index = faiss.IndexFlatL2(1024)
embeddings = HuggingFaceEmbeddings(model_name='/model/moka-ai/m3e-large')
db = FAISS(embedding_function=embeddings.client.encode,index=index,docstore=docstore1,index_to_docstore_id={})

psqldb = PostgresDB("192.168.0.148", "vecdoc", "vecdoc", "vecdoc")
psqldb.connect()
#将当前向量库中的数据全部导出到新的向量库中
db_index = 0
page_size = 500
print(db.index.ntotal)
while True:
    query = f"SELECT text,paragraph_id FROM vec_txt order by vector_id limit %s offset %s" % (page_size,db_index)
    psqldb.execute(query)
    questions = psqldb.fetchall()
    if len(questions) <= 0:
        break
    db_index+=page_size
    list_of_documents = []
    for question in questions:
        query = f"SELECT text FROM txt_doc WHERE paragraph_id = %s" % (question[1])
        psqldb.execute(query)
        paragraph = psqldb.fetchall()[0]
        list_of_documents.append(Document(page_content=question[0], metadata=dict(paragraph=paragraph[0])))
    db.add_documents(list_of_documents)
    print(db.index.ntotal)
    # break
    
#导出完毕，查询向量库测试
results_with_scores = db.similarity_search_with_score("忘记密码怎么办")
page_ids = []
for doc, score in results_with_scores:
    if doc.metadata["page"] in page_ids:
        continue
    else:
        page_ids.append(doc.metadata["page"])
        print(f"Content: {doc.page_content}, Metadata: {doc.metadata}, Score: {score}")
# faiss.write_index(db.index, "./faiss_store/know.faiss")     
#将向量库持久化到本地  
db.save_local(folder_path="./faiss_store",index_name="know")



